#define SYSTEM_NDS  1

#include <nds.h>

#include "ProcessorDefine.h"
#include "FIFO.h"

#if PROC_ARM9
#include "DebugStr.h"
#include "GameSounds.h"
#endif

#if PROC_ARM7
#include "../Source/Sound.h"
#endif

#define null        0

int FIFO_LastMessageSent = 0;

void ARM7FIFO_Setup()
{
    irqSet(IRQ_FIFO_NOT_EMPTY, ARM7FIFO_Handler);
    irqEnable(IRQ_FIFO_NOT_EMPTY);
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR | IPC_FIFO_RECV_IRQ; 
}

void ARM7FIFO_Handler()
{
#if PROC_ARM7
    while(!(REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY))
    {
        FIFO_Message* message = (FIFO_Message*)REG_IPC_FIFO_RX;
        HandleFIFOMessage(message);
    }
#endif
}

void ARM9FIFO_Setup()
{
    irqSet(IRQ_FIFO_NOT_EMPTY, ARM9FIFO_Handler);
    irqEnable(IRQ_FIFO_NOT_EMPTY);
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR | IPC_FIFO_RECV_IRQ;
}

void ARM9FIFO_Handler()
{
#if PROC_ARM9
    while(!(REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY))
    {
        FIFO_Message* message = (FIFO_Message*)REG_IPC_FIFO_RX;
        HandleFIFOMessage(message);
    }
#endif
}

FIFO_Message* FIFO_GetMessageContainer()
{
    // just don't send if the queue is full...
    // doing this to not crash dualis, but might drop sounds on hardware.
    if(REG_IPC_FIFO_CR & IPC_FIFO_SEND_FULL)
        return null;
    
    // stall while the fifo queue is full.
    while(REG_IPC_FIFO_CR & IPC_FIFO_SEND_FULL);

    // if there's an error, wipe out the queue.
    if(REG_IPC_FIFO_CR & IPC_FIFO_ERROR)
        REG_IPC_FIFO_CR |= IPC_FIFO_SEND_CLEAR;

    FIFO_Message* FIFO_NextMessage = &FIFO_MessageArray[FIFO_LastMessageSent];

    FIFO_LastMessageSent = (FIFO_LastMessageSent+1) % FIFO_MAX_MESSAGES;

    return FIFO_NextMessage; //&FIFO_MessageRam[FIFO_LastMessageSent];
}

void FIFO_SendMessage(FIFO_Message* command)
{
    REG_IPC_FIFO_TX = (u32)command;
}
