//---------------------------------------------------------------------------------
//    Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)
//---------------------------------------------------------------------------------
#include <nds.h>
#include <stdlib.h>

#include <nds/bios.h>
#include <nds/arm7/touch.h>
#include <nds/arm7/clock.h>

#include <dswifi7.h>

#include "FIFO.h"
#include "Sound.h"
#include "debugstr.h"
#include "Microphone.h"

int _EMULATOR = 1;

int MyMIC_Level = 20;
int MyMIC_On = 0;

int Wifi_Enabled = 0;

void VblankHandler(void)
{
    static int heartbeat = 0;

    uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0, batt=0, aux=0;
    int t1=0, t2=0;
    uint32 temp=0;
    uint8 ct[sizeof(IPC->curtime)];
    u32 i;

    // Update the heartbeat
    heartbeat++;

    // Read the touch screen
    but = REG_KEYXY;

    if(!(but & (1<<6)))
    {
        touchPosition tempPos = touchReadXY();

        x = tempPos.x;
        y = tempPos.y;
        xpx = tempPos.px;
        ypx = tempPos.py;
    }

    z1 = touchRead(TSC_MEASURE_Z1);
    z2 = touchRead(TSC_MEASURE_Z2);

    batt = touchRead(TSC_MEASURE_BATTERY);
    aux  = touchRead(TSC_MEASURE_AUX);

    // Read the time
    rtcGetTime((uint8 *)ct);
    BCDToInteger((uint8 *)&(ct[1]), 7);

    // Read the temperature
    temp = touchReadTemperature(&t1, &t2);

    // Update the IPC struct
    IPC->heartbeat  = heartbeat;
    IPC->buttons        = but;
    IPC->touchX         = x;
    IPC->touchY         = y;
    IPC->touchXpx       = xpx;
    IPC->touchYpx       = ypx;
    IPC->touchZ1        = z1;
    IPC->touchZ2        = z2;
    IPC->battery        = batt;
    IPC->aux            = aux;

    for(i=0; i<sizeof(ct); i++)
    {
        IPC->curtime[i] = ct[i];
    }

    IPC->temperature = temp;
    IPC->tdiode1 = t1;
    IPC->tdiode2 = t2;

    CheckIfSoundsEnded();

    if(MyMIC_On)
    {
        static int lastmicsound;
        int micsound;
        micsound = abs(tob_MIC_ReadData8() - 128);
        if(micsound > MyMIC_Level && lastmicsound > MyMIC_Level)
        {
            FIFO_Message* message = FIFO_GetMessageContainer();
            if(message)
            {
                message->command = MESSAGE_MIC_MADESOUND;
                message->rate = micsound;
                FIFO_SendMessage(message);
            }
        }
        lastmicsound = micsound;
    }

    if(Wifi_Enabled)
	    Wifi_Update(); // update wireless in vblank

    ////sound code
    //TransferSound* snd = IPC->soundData;
    //IPC->soundData = 0;

    //if(snd != 0)
    //{
    //    for(i=0; i<snd->count; i++)
    //    {
    //        s32 chan = getFreeSoundChannel();

    //        if(chan >= 0)
    //        {
    //            startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
    //        }
    //    }
    //}
}

int main(int argc, char ** argv)
{
    // Reset the clock if needed
    rtcReset();

    //enable sound
    powerON(POWER_SOUND);
    SOUND_CR = SOUND_ENABLE | SOUND_VOL(127);
    IPC->soundData = 0;

    irqInit();
    irqSet(IRQ_VBLANK, VblankHandler);
    irqEnable(IRQ_VBLANK);

    ARM7FIFO_Setup();
    SOUND_BIAS = 0x200;
    InitSounds();

    tob_MIC_On();

    int i;
    for(i=0; i<50; i++)
        tob_MIC_ReadData8();

    //DEBUG_Print("ARM7 - debug");

    // Keep the ARM7 out of main RAM
    while(1)
    {
        swiWaitForVBlank();
    }
}
