#include <nds.h>

#include <dswifi7.h>

#include "FIFO.h"
#include "Sound.h"
#include "Microphone.h"

extern int MyMIC_Level;
extern int MyMIC_On;
extern int Wifi_Enabled;

// callback to allow wifi library to notify arm9
void arm7_synctoarm9() // send fifo message
{
    FIFO_Message* message = FIFO_GetMessageContainer();
    if(message)
    {
        message->command = MESSAGE_WIFI_SYNC;
        FIFO_SendMessage(message);
    }
}

void HandleFIFOMessage(FIFO_Message* message)
{
    if(message == 0)
        return;
    
    switch(message->command)
    {
    case MESSAGE_PLAY_SOUND:
        HandleSoundMessage(message);
        break;

    case MESSAGE_MIC_TURNON:
        tob_MIC_On();
        MyMIC_Level = message->rate;
        MyMIC_On = 1;
        break;

    case MESSAGE_MIC_TURNOFF:
        tob_MIC_Off();
        MyMIC_On = 0;
        break;

    case MESSAGE_WIFI_INIT:
        Wifi_Enabled = 1;
        Wifi_Init(message->len);
        Wifi_SetSyncHandler(arm7_synctoarm9); // allow wifi lib to notify arm9
        break;

    case MESSAGE_WIFI_SYNC:
       Wifi_Sync();
       break;

    default:
        break;
    }
}

struct SoundInfo
{
    u8 playing;
    const void* data;
    u32 len;
    u32 rate;
    u8 vol;
    u8 pan;
    u8 format;
    u8 channel;
};

SoundInfo CurrentSoundsPlaying[16];

void InitSounds()
{
    int i;
    for(i=0; i<16; i++)
        CurrentSoundsPlaying[i].playing = 0;
}

void CheckIfSoundsEnded()
{
    int i;
    for(i=0; i<16; i++)
    {
        if(CurrentSoundsPlaying[i].playing == 1)
        {
            // if sound ended;
            if( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0)
            {
                FIFO_Message* message = FIFO_GetMessageContainer();
                if(message)
                {
                    message->command = MESSAGE_SOUND_ENDED;
                    message->channel = i;
                    FIFO_SendMessage(message);
                }
            }
        }
    }
}

void HandleSoundMessage(FIFO_Message* message)
{
    s8 channel;

    if(message->channel > 15)
        channel = getFreeSoundChannel();
    else
        channel = message->channel;

    if(channel >= 0)
    {
        startSound(message->rate,
                   message->data,
                   message->len,
                   channel,
                   message->vol,
                   message->pan,
                   message->format);
    }
}

void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format)
{
    CurrentSoundsPlaying[channel].playing = 1;

    SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
    SCHANNEL_SOURCE(channel) = (u32)data;
    SCHANNEL_LENGTH(channel) = bytes >> 2;
    SCHANNEL_CR(channel) = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | 
                        SOUND_PAN(pan) | (format << 29);
}

s32 getFreeSoundChannel()
{
    int i;
    // TODO: fix this //HACKED this since first 2 channels are reserved for music...
    for(i=2; i<16; i++)
    {
        if( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
    }
    return -1;
}

