#include "MySystem.h"
#include "BitmapDraw.h"

void BitmapDrawRect(char* bitmap, u16 bmsizex, u16 bmsizey, int sx, int sy, u16 w, u16 h, u8 color)
{
    u16 x, y;
    int ex = sx+w;
    int ey = sy+h;

    if(sx < 0) sx = 0;
    if(sy < 0) sy = 0;
    if(ex > bmsizex) ex = bmsizex;
    if(ey > bmsizey) ey = bmsizey;

    if(ex < 0 || ey < 0)
        return;

    for(y = sy; y < ey; y++)
    {
        for(x = sx; x < ex; x++)
        {
            bitmap[y*bmsizex+x] = color;
        }
    }
}

void BitmapDrawRectOutline(char* bitmap, u16 bmsizex, u16 bmsizey, int sx, int sy, u16 w, u16 h, u8 color)
{
    u16 x, y;
    int ex = sx+w;
    int ey = sy+h;

    if(sx < 0) sx = 0;
    if(sy < 0) sy = 0;
    if(ex > bmsizex) ex = bmsizex;
    if(ey > bmsizey) ey = bmsizey;

    if(ex < 0 || ey < 0)
        return;

    for(x = sx; x < ex; x++)
    {
        bitmap[sy*bmsizex+x] = color;
        bitmap[ey*bmsizex+x] = color;
    }

    for(y = sy; y < ey; y++)
    {
        bitmap[y*bmsizex+sx] = color;
        bitmap[y*bmsizex+ex] = color;
    }
}

void BitmapDrawLine(char* bitmap, u16 bmsizex, u16 bmsizey, int x1, int y1, int x2, int y2, u8 color, float perc1, float perc2)
{
    // pixels are double height... so make sure the start/end pos's are 2 away from the bottom.
    if(x1 < 0) x1 = 0;
    if(y1 < 0) y1 = 0;
    if(x1 >= bmsizex-1) x1 = bmsizex-2;
    if(y1 >= bmsizey-1) y1 = bmsizey-2;

    if(x2 < 0) x2 = 0;
    if(y2 < 0) y2 = 0;
    if(x2 >= bmsizex-1) x2 = bmsizex-2;
    if(y2 >= bmsizey-1) y2 = bmsizey-2;

    int xinc1;
    int xinc2;
    int yinc1;
    int yinc2; 
    int den;
    int num;
    int numadd;
    int numpixels;

    int deltax = abs(x2 - x1);        // The difference between the x's
    int deltay = abs(y2 - y1);        // The difference between the y's
    int x = x1;                       // Start x off at the first pixel
    int y = y1;                       // Start y off at the first pixel

    if(x2 >= x1)                 // The x-values are increasing
    {
        xinc1 = 1;
        xinc2 = 1;
    }
    else                          // The x-values are decreasing
    {
        xinc1 = -1;
        xinc2 = -1;
    }

    if(y2 >= y1)                 // The y-values are increasing
    {
        yinc1 = 1;
        yinc2 = 1;
    }
    else                          // The y-values are decreasing
    {
        yinc1 = -1;
        yinc2 = -1;
    }

    if(deltax >= deltay)         // There is at least one x-value for every y-value
    {
        xinc1 = 0;                  // Don't change the x when numerator >= denominator
        yinc2 = 0;                  // Don't change the y for every iteration
        den = deltax;
        num = deltax / 2;
        numadd = deltay;
        numpixels = deltax;         // There are more x-values than y-values
    }
    else                          // There is at least one y-value for every x-value
    {
        xinc2 = 0;                  // Don't change the x for every iteration
        yinc1 = 0;                  // Don't change the y when numerator >= denominator
        den = deltay;
        num = deltay / 2;
        numadd = deltax;
        numpixels = deltay;         // There are more y-values than x-values
    }

    int startpixel = (int)(numpixels * perc1);
    int endpixel = (int)(numpixels * perc2);

    for(int curpixel = 0; curpixel <= numpixels; curpixel++)
    {
        if(curpixel >= startpixel && curpixel <= endpixel)
        {
            bitmap[ y   *bmsizex + x  ] = color;
            bitmap[(y+1)*bmsizex + x  ] = color;
            bitmap[ y   *bmsizex + x+1] = color;
            bitmap[(y+1)*bmsizex + x+1] = color;
        }
        //PutPixel(x, y);             // Draw the current pixel
        num += numadd;              // Increase the numerator by the top of the fraction
        if (num >= den)             // Check if numerator >= denominator
        {
            num -= den;               // Calculate the new numerator value
            x += xinc1;               // Change the x as appropriate
            y += yinc1;               // Change the y as appropriate
        }
        x += xinc2;                 // Change the x as appropriate
        y += yinc2;                 // Change the y as appropriate
    }
}
/*
    int		dx,		//deltas
            dy,
            dx2,	//scaled deltas
            dy2,
            ix,		//increase rate on the x axis
            iy,		//increase rate on the y axis
            err,	//the error term
            i;		//looping variable

	int pitch = bmsizex; //lpitch;

	// identify the first pixel
	//ushort *ptr_vid = lpscreen + x1 + (y1 * (pitch >> 1));
    char* ptr_vid = &bitmap[y1*pitch + x1];

	// difference between starting and ending points
	dx = x2 - x1;
	dy = y2 - y1;

	// calculate direction of the vector and store in ix and iy
	if (dx >= 0)
		ix = 1;

	if (dx < 0)
	{
		ix = -1;
		dx = abs(dx);
	}

	if (dy >= 0)
		iy = (pitch >> 1);

	if (dy < 0)
	{
		iy = -(pitch >> 1);
		dy = abs(dy);
	}

	// scale deltas and store in dx2 and dy2
	dx2 = dx * 2;
	dy2 = dy * 2;
        
	if (dx > dy)	// dx is the major axis
	{
		// initialize the error term
		err = dy2 - dx;

		for (i = 0; i <= dx; i++)
		{
			*ptr_vid = color;
			if (err >= 0)
			{
				err -= dx2;
				ptr_vid += iy;
			}
			err += dy2;
			ptr_vid += ix;
		}
	}
	
	else 		// dy is the major axis
	{
		// initialize the error term
		err = dx2 - dy;

		for (i = 0; i <= dy; i++)
		{
			*ptr_vid = color;
			if (err >= 0)
			{
				err -= dy2;
				ptr_vid += ix;
			}
			err += dx2;
			ptr_vid += iy;
		}
	}
} // end of Line(...
*/














/*
void BitmapDrawLine(char* bitmap, u16 bmsizex, u16 bmsizey, int sx, int sy, int ex, int ey, u8 color)
{
    if(sx < 0) sx = 0;
    if(sy < 0) sy = 0;
    if(sx > bmsizex) sx = bmsizex;
    if(sy > bmsizey) sy = bmsizey;

    if(ex < 0) ex = 0;
    if(ey < 0) ey = 0;
    if(ex > bmsizex) ex = bmsizex;
    if(ey > bmsizey) ey = bmsizey;

    int x = 0;
    int y = 0;

    float flenx = ex-sx;
    float fleny = ey-sy;

    if(sx > ex)
    {
        x = ex;
        ex = sx;
        sx = x;
    }
    if(sy > ey)
    {
        y = ey;
        ey = sy;
        sy = y;
    }

    int lenx = ex-sx;
    int leny = ey-sy;

    int xlonger = 0;
    if(lenx > leny)
        xlonger = 1;

    if(lenx == 0 && leny == 0)
        return;

    if(xlonger == 0)
    {
        float inc = flenx/fleny;
        float fx = sx;
        for(y=sy; y<=ey; y++)
        {
            fx += inc;
            x = (int)fx;
            bitmap[y*bmsizex+x] = color;
        }
    }
    else
    {
        float inc = fleny/flenx;
        float fy = sy;
        for(x=sx; x<=ex; x++)
        {
            fy += inc;
            y = (int)fy;
            bitmap[y*bmsizex+x] = color;
        }
    }
}
*/
