#include "MySystem.h"

#include "BaseObject.h"
#include "MyDMA.h"

#include "BitmapDraw.h"
#include "SpriteRamHandler.h"
#include "ObjectManager.h"

BaseObject::BaseObject() :
    m_Delay(0),
    m_Frame(0),
    m_AnimSpeed(5),
    m_FlipHorizontal(0),
    m_StackedSpriteHack(0),
    m_ID(0)
{
    //memset(this, 0, sizeof(BaseObject));

    m_Type = OBJECTTYPE_None;

    m_TI = 0;
    m_Animated = 0;

    m_CurrentAnim = OBJECTANIM_Idle;
    m_LastSeenAnim = OBJECTANIM_Invalid;

    m_PosX = -64;
    m_PosY = -64;
    m_PosZ = 0;

    m_OverrideSortPos = 9999;

    m_LastX = -64;
    m_LastY = -64;

    m_MovedThisFrame = 0;

    m_SizeX = 0;
    m_SizeY = 0;

    m_OffsetX = 0;
    m_OffsetY = 0;

    m_Transparent = 0;
    
    m_Health = 1000;
    m_MaxHealth = 1000;

    m_ToBeDeleted = 0;
}

BaseObject::~BaseObject()
{
    // remove this object from any list it's in when it's deleted.
    if(Prev)
        Remove();
}

void BaseObject::SetTileInfo(TilesInfo* ti, int sprite, int animated)
{
    m_TI = ti;
    if(m_TI->numFrames > 1)
        m_Animated = 1;

    if(animated != -1)
        m_Animated = animated;

    m_Frame = sprite;

    // TODO: make sure sizeX and sizeY are the biggest possible for this sprite.
    m_SizeX = m_TI->MinWidth[0];
    m_SizeY = m_TI->MinHeight[0];

    m_OffsetX = m_TI->OrigWidth[0] / 2; // m_SizeX / 2;
    m_OffsetY = m_TI->OrigHeight[0];

    if(m_StackedSpriteHack)
    {
        m_SizeY = m_TI->MinHeight[0] + m_TI->MinHeight[m_TI->numFrames/2];
        m_OffsetY = m_TI->OrigHeight[0] + m_TI->OrigHeight[m_TI->numFrames/2];
    }
}

void BaseObject::Tick()
{
    if(m_Health > m_MaxHealth)
        m_Health = m_MaxHealth;
}

void BaseObject::SetCurrentAnim(int action)
{
    m_CurrentAnim = action;
    m_Frame = m_TI->AnimFrameOffset[m_CurrentAnim];
    m_Delay = 0;
}

void BaseObject::UpdateSprite(int uselastseenanim)
{
    int posx = (int)m_PosX;
    int posy = (int)m_PosY;

    //set up our sprites OAM entry attributes
    int scrx = posx - m_OffsetX + BOARD_OFFSET_X;
    int scry = posy - m_OffsetY - (int)m_PosZ + BOARD_OFFSET_Y;

    int dirfaced = OBJECTDIR_Up;
    int palettenum = 0;

    int screen = 0;
    if(scry < -SCREEN_SEP)
    {
        screen = 1;
        scry += LEVEL2_Z;
    }

    if(m_Type == OBJECTTYPE_Player)
    {
        dirfaced = ((PlayerObject*)this)->QueryDirectionFaced();
        palettenum = ((PlayerObject*)this)->m_MachineNum+1; //QueryPlayerNum();
    }

    // don't draw the elevator or the guy if they're 26 units below the elevator on the top screen.
    if(m_Type == OBJECTTYPE_Elevator || m_Type == OBJECTTYPE_Player)
    {
        if(screen == 1 && m_PosZ < LEVEL2_Z - 20)
            return;
    }

    if(m_Animated)
    {
        m_Delay++;

        if(m_Delay >= m_AnimSpeed)
        {
            m_Delay = 0;
            m_Frame++;

            int anim = m_CurrentAnim;
            if(uselastseenanim && m_LastSeenAnim != OBJECTANIM_Invalid)
                anim = m_LastSeenAnim;

            if(m_Type != OBJECTTYPE_Player)
            {
                // code for things with 1 direction.
                //anim /= 3;
            }
            else
            {
                if(dirfaced == OBJECTDIR_Down)
                {
                }
                else if(dirfaced == OBJECTDIR_Right || dirfaced == OBJECTDIR_Left)
                    anim += 3;
                else
                    anim += 6;
            }

            if(m_Frame >= m_TI->AnimFrameOffset[anim] + m_TI->numAnimFrames[anim])
                m_Frame = m_TI->AnimFrameOffset[anim];

            if(m_Frame < m_TI->AnimFrameOffset[anim])
                m_Frame = m_TI->AnimFrameOffset[anim];
        }
    }

    int sortposy = (int)m_PosY;

    if(m_OverrideSortPos != 9999)
        sortposy = m_OverrideSortPos;
    //if(m_StackedSpriteHack)
    //{
    //    // top
    //    g_MainSprites.AddSprite(sortposy, m_TI, m_Frame+m_TI->numFrames/2,
    //                            scrx, scry+m_TI->OffsetY[m_Frame], m_Transparent, m_PlayerNum, 3, 0, 0);
    //    // bottom
    //    g_MainSprites.AddSprite(sortposy, m_TI, m_Frame,
    //                            scrx, scry+m_TI->MinHeight[m_Frame]+m_TI->OffsetY[m_Frame], m_Transparent, m_PlayerNum, 3, 0, 0);
    //}
    //else
    {
        int fliphor = 0;
        if(m_FlipHorizontal || dirfaced == OBJECTDIR_Left)
            fliphor = 1;

        if(screen == 0)
        {
            g_MainSprites.AddSprite(sortposy, m_TI, m_Frame, scrx, scry, m_Transparent, palettenum, 3, fliphor, 0);
        }
        else
        {
            g_SubSprites.AddSprite(sortposy, m_TI, m_Frame, scrx, scry, m_Transparent, palettenum, 3, fliphor, 0);
        }
    }

    AddExtraSprites(sortposy, scrx, scry);

    m_LastX = m_PosX;
    m_LastY = m_PosY;
}

void BaseObject::SetTransparent(int transparent)
{
    m_Transparent = transparent;
}

void BaseObject::MoveAbs(float x, float y)
{
    m_LastX = m_PosX;
    m_LastY = m_PosY;

    m_PosX = x;
    m_PosY = y;
}

void BaseObject::MoveRelative(float x, float y)
{
    m_LastX = m_PosX;
    m_LastY = m_PosY;

    m_PosX += x;
    m_PosY += y;
}

void BaseObject::SetPosZ(float z)
{
    m_PosZ = z;
    if(m_PosZ < 0)
        m_PosZ = 0;
    if(m_PosZ > LEVEL2_Z)
        m_PosZ = LEVEL2_Z;
}

void BaseObject::AddPosZ(float z)
{
    m_PosZ += z;
    if(m_PosZ < 0)
        m_PosZ = 0;
    if(m_PosZ > LEVEL2_Z)
        m_PosZ = LEVEL2_Z;
}

void BaseObject::AddHealth(float diff)
{
    m_Health += diff;
    if(m_Health > m_MaxHealth)
        m_Health = m_MaxHealth;

    if(m_Health < 0)
        m_Health = -1;

    //if(m_Health < 0)
    //    MarkToBeDeleted();
}




