#ifndef __BASEOBJECT_H__
#define __BASEOBJECT_H__

enum ObjectType
{
    OBJECTTYPE_None = 0,
    OBJECTTYPE_Player,
    OBJECTTYPE_BreakableBlock,
    OBJECTTYPE_Block,
    OBJECTTYPE_Bomb,
    OBJECTTYPE_FakeBomb,
    OBJECTTYPE_Flame,
    OBJECTTYPE_Elevator,
    OBJECTTYPE_PowerUp,
    OBJECTTYPE_NumTypes,
};

enum
{
    OBJECTANIM_Idle,
    OBJECTANIM_Invalid,
};

class BaseObject : public CPPListNode
{
protected:
    ObjectType m_Type;

    TilesInfo* m_TI;
    int m_Animated;

    int m_CurrentAnim;
    int m_LastSeenAnim;

    u16 m_Delay;
    u8 m_Frame;
    u8 m_AnimSpeed;

    bool m_FlipHorizontal;

    bool m_StackedSpriteHack; // set this to 1 to have two sprites stack one on another.

    float m_PosX;
    float m_PosY;
    float m_PosZ;
    float m_LastX;
    float m_LastY;

    int m_OverrideSortPos;

    int m_MovedThisFrame;
    
    u8 m_SizeX;
    u8 m_SizeY;
    u8 m_OffsetX; // half sizeX... so world pos is the bottom center of tile
    u8 m_OffsetY; // sizeY...

    u8 m_Transparent;

    float m_Health;
    u16 m_MaxHealth;

    int m_ToBeDeleted;

public:
    u16 m_ID; // Complete hack to id object on the network.

    BaseObject();
    virtual ~BaseObject();

    void SetTileInfo(TilesInfo* ti, int sprite, int animated = -1);
    TilesInfo* QueryTileInfo() { return m_TI; }

    virtual void Tick();
    void UpdateSprite(int uselastseenanim = 0);
    virtual void AddExtraSprites(int sortposy, int x, int y) {}

    void SetTransparent(int transparent);

    void SetCurrentAnim(int action);
    int QueryCurrentAnim() { return m_CurrentAnim; }

    virtual void MoveAbs(float x, float y);
    virtual void MoveRelative(float x, float y);
    void SetPosZ(float z);
    void AddPosZ(float z);

    ObjectType QueryObjectType() { return m_Type; }

    void SetFlipHorizontal(bool flip) { m_FlipHorizontal = flip; }
    bool GetFlipHorizontal() { return m_FlipHorizontal; }

    int QueryPosX() { return (int)m_PosX; }
    int QueryPosY() { return (int)m_PosY; }
    int QueryPosZ() { return (int)m_PosZ; }
    int QueryLastX() { return (int)m_LastX; }
    int QueryLastY() { return (int)m_LastY; }

    int QuerySizeX() { return m_SizeX; }
    int QuerySizeY() { return m_SizeY; }

    virtual void AddHealth(float diff);
    float QueryHealth() { return m_Health; }
    u16 QueryMaxHealth() { return m_MaxHealth; }
    int IsDamaged() { return m_MaxHealth - (int)m_Health; }

    void MarkToBeDeleted() { m_ToBeDeleted = 1; }
    int QueryToBeDeleted() { return m_ToBeDeleted; }
};

#endif //__BASEOBJECT_H__
