#include "MySystem.h"

#include "BombGraphics.h"
#include "WIFIMenu.h"
#include "ScoreBox.h"
#include "GameSounds.h"

ObjectManager* g_pObjectManager;
GameBoard* g_pGameBoard;
PlayerObject* g_pPlayers[MAX_PLAYERS];
char g_PlayerName[MAX_PLAYERS][20];     // based on the machine number
int g_PlayerWins[MAX_PLAYERS];          // based on the machine number
int g_PlayerBeingControlled;
int g_GameStarted = 0;
int g_RoundStarted = 0;
int g_WaitingForGameState = 0;
int g_MachineNumber = -1;
int g_MovePacketFreq = DEFAULTMOVEPACKETFREQ;

void BombMain()
{
    int i;

    srand(GameOptions.InitialSeed);

    g_GameStarted = 1;
    g_RoundStarted = 0;
    g_WaitingForGameState = 0;
    int HostMachine = 0;
    int OnlineMatch = 0;
    int TickRoundStarted = 0;
    int TickLastFloodSent = 0;

    InitGraphicModes();
    LoadBombGuyGraphics();
    SetupScoreBox();

    g_pObjectManager = MyNew(ObjectManager);
    g_pGameBoard = MyNew(GameBoard);

    MarkAllChannelsClear();

    for(i=0; i<MAX_PLAYERS; i++)
    {
        g_pPlayers[i] = NULL;
        g_PlayerName[i][0] = 0;
        g_PlayerWins[i] = 0;
    }

    if(GameClient || GameServer)
        OnlineMatch = 1;

    if(!GameClient)
    {
        HostMachine = 1;
        g_pGameBoard->Setup();
    }
    else
    {
        MessageRequestGameState* mess = MyNew MessageRequestGameState;
        mess->Send();
        SAFE_REDUCEREFANDDELETE(mess);

        g_WaitingForGameState = 1;
    }

#if SYSTEM_NDS
    //for(i=0; i<PersonalData->nameLen; i++)
    //    g_PlayerName[0][i] = PersonalData->name[i];
    //g_PlayerName[0][i] = 0;
    g_FrameCount = 0;
#endif //SYSTEM_NDS

    g_PlayerBeingControlled = -1;
    int LastTickSent = 0;

    //lcdSwap();

    int keepplaying = 1;
    int connectionlost = 0;
    int askedforplayer = 0;

    while(keepplaying)
    {
        ClearScoreBoxMap(NULL);
        ClearSoundsPlayedThisFrame();

        if(GameServer)
        {
            GameServer->Tick();
        }

        if(GameClient)
        {
            if( GameClient->Tick() )
            {
                // lost connection.
                keepplaying = 0;
                connectionlost = 1;
                continue;
            }
        }

        scanKeys();
        
        if(keysUp() & KEY_SELECT)
        {
            keepplaying = 0;
            continue;
        }

        if(g_WaitingForGameState)
        {
            // TODO: add a graphic for this.
            PrintToScreen("Waiting for gamestate", 21, 0);

            g_RoundStarted = 0;
            askedforplayer = 0;
        }
        else
        {
            if(!g_RoundStarted)
            {
                // TODO: add a graphic for this.
                if(GameClient)
                    PrintToScreen("Waiting for round to start", 21, 0);
                else
                    PrintToScreen("Press 'A' to start round", 21, 0);
                ShowScoreBox();

                // force all machines to add a player
                if(g_PlayerBeingControlled == -1)
                {
                    if(!askedforplayer)
                    {
                        askedforplayer = 1;
                        MessageAddPlayer* mess = MyNew MessageAddPlayer();

                        for(i=0; i<PersonalData->nameLen; i++)
                            mess->name[i] = PersonalData->name[i];
                        mess->name[i] = 0;

                        mess->Send();
                        SAFE_REDUCEREFANDDELETE(mess);
                    }
                }
                else
                {
                    // TODO: undo this hack.  autostart a round on pc's after 5 seconds.
                    if(HostMachine && keysUp() & KEY_A)
                    {
                        int numplayers = 0;
                        for(i=0; i<MAX_PLAYERS; i++)
                            if(g_pPlayers[i])
                                numplayers++;

                        if(!OnlineMatch || numplayers > 1)
                        {
                            MessageStartGame* mess = MyNew(MessageStartGame);
                            mess->Send();
                            SAFE_REDUCEREFANDDELETE(mess);
                            TickRoundStarted = g_FrameCount;
                            TickLastFloodSent = g_FrameCount;
                        }
                    }
                }
            }
            else
            {
                // set this for offline play.
                g_RoundStarted = 1;

                // setup to ask for a player when the next round starts.
                askedforplayer = 0;

                if(g_PlayerBeingControlled != -1)
                {
                    PrintToScreen("playing", 21, 0);

                    g_pPlayers[g_PlayerBeingControlled]->SetKeysHeld( keysHeld() );
                    for(i=0; i<MAX_PLAYERS; i++)
                        if(g_pPlayers[i])
                            g_pPlayers[i]->ProcessKeys();

                    if(g_FrameCount - LastTickSent > g_MovePacketFreq)
                    {
                        if(GameClient)
                        {
                            LastTickSent = g_FrameCount;
                            g_pPlayers[g_PlayerBeingControlled]->SendMovePacket(g_PlayerBeingControlled);
                        }
                        else if(GameServer)
                        {
                            LastTickSent = g_FrameCount;
                            MessagePlayerInfo* mess = MyNew(MessagePlayerInfo);
                            for(i=0; i<MAX_PLAYERS; i++)
                            {
                                if(g_pPlayers[i])
                                    mess->Fill(i, (float)g_pPlayers[i]->QueryPosX(), (float)g_pPlayers[i]->QueryPosY(), (float)g_pPlayers[i]->QueryPosZ(), g_pPlayers[i]->GetKeysHeld());
                                else
                                    mess->Fill(i, 0, 0, 0, 0);
                            }
                            mess->Send();
                            SAFE_REDUCEREFANDDELETE(mess);                            
                        }
                    }
                }
                else
                {
                    // TODO: add a graphic for this.
                    PrintToScreen("wait for next round", 21, 0);
                }
            }
        }

        g_MainSprites.ClearSprites();
        g_SubSprites.ClearSprites();

        g_pObjectManager->TickAllObjects();

        int numplayers = 0;
        int lastplayer = -1;
        for(i=0; i<MAX_PLAYERS; i++)
        {
            if(g_pPlayers[i])
            {
                if(g_pPlayers[i]->QueryToBeDeleted())
                {
                    if(g_PlayerBeingControlled == i)
                        g_PlayerBeingControlled = -1;
                    g_pPlayers[i] = NULL;
                }
                else
                {
                    numplayers++;
                    lastplayer = i;
                }
            }
        }

        // flood gameplay area with bombs.
        int TicksSinceRoundStarted = g_FrameCount - TickRoundStarted;
        int TicksSinceLastFlood = g_FrameCount - TickLastFloodSent;
        int TimeOut = 60*90;
        int Tween = 60*7;
        if((!OnlineMatch || GameServer) && g_RoundStarted && TicksSinceRoundStarted > TimeOut && TicksSinceLastFlood > Tween)
        {
            TickLastFloodSent = g_FrameCount;
            MessageBombFlood* mess = MyNew(MessageBombFlood);
            mess->map = rand()%2;
            mess->side = rand()%2;
            mess->Send();
            SAFE_REDUCEREFANDDELETE(mess);
        }

        // check if the game ended.
        if(!OnlineMatch || GameServer)
        {
            // for offline games just respawn the player... it's just for testing.
            if( !OnlineMatch && numplayers == 0 )
            {
                // then send a new board layout(which will delete all players).
                g_pGameBoard->Setup();
                g_pGameBoard->SendBoardLayout(-1);

                if(g_RoundStarted || (numplayers == 0))
                {
                    g_RoundStarted = 0;
                    askedforplayer = 0;
                }
            }

            // check if someone won an online game and generate a new board.
            if( g_RoundStarted && OnlineMatch && numplayers <= 1 )
            {
                if(lastplayer != -1)
                {
                    if(g_pPlayers[lastplayer]->QueryCurrentAnim() == PLAYERANIM_Win)
                    {
                        int mac = g_pPlayers[lastplayer]->m_MachineNum + 1;
                        if(mac >= 0 && mac < MAX_PLAYERS)
                            g_PlayerWins[mac]++;
                    }
                }

                g_PlayerBeingControlled = -1;
                g_RoundStarted = 0;
                g_WaitingForGameState = 1;
                askedforplayer = 0;

                // send a round ended message.
                MessageRoundEnded* mess = MyNew(MessageRoundEnded);
                mess->Send();
                SAFE_REDUCEREFANDDELETE(mess);

                // then send a new board layout(which will delete all players).
                g_pGameBoard->Setup();
                g_pGameBoard->SendBoardLayout(-1);
            }
        }

        g_pObjectManager->DeleteDeadObjects();
        g_pObjectManager->UpdateObjectSprites();

        g_MainSprites.UpdateAll(OAM);
        g_SubSprites.UpdateAll(OAM_SUB);

        swiWaitForVBlank();
    }

    //lcdSwap();

    SAFE_DELETE(g_pGameBoard);
    SAFE_DELETE(g_pObjectManager);

    SAFE_DELETE(GameServer);
    SAFE_DELETE(GameClient);

    g_GameStarted = 0;
    g_MachineNumber = -1;

    MenuInitSystem();
}
