#include "MySystem.h"

#include "WifiMenu.h"

void ProcessGameMessages(NetMessage* message, int conn)
{
    if(!g_GameStarted)
        return;

    switch(message->type)
    {
    case NM_RequestGameState:
        {
            if(GameClient)
                break;

            // if this message isn't from a client ignore it.
            if(conn == -1)
                break;

            int i;

            // only new clients request this, so reset his score/name
            g_PlayerWins[conn+1] = 0;
            g_PlayerName[conn+1][0] = 0;

            // send the board layout and the current framecount.
            g_pGameBoard->SendBoardLayout(conn);
            for(i=0; i<MAX_PLAYERS; i++)
            {
                if(g_pPlayers[i])
                {
                    MessagePlayerJoined* messtosend = MyNew MessagePlayerJoined();
                    messtosend->player = i;
                    messtosend->machinewhocontrolsit = g_pPlayers[i]->m_MachineNum;
                    messtosend->score = g_PlayerWins[g_pPlayers[i]->m_MachineNum+1];
                    strcpy(messtosend->name, g_PlayerName[g_pPlayers[i]->m_MachineNum+1]);
                    messtosend->Send(conn);
                    SAFE_REDUCEREFANDDELETE(messtosend);
                }
            }
        }
        break;

    case NM_GameInfo:
        {
            g_WaitingForGameState = 0;

            // clear out existing playerobjects;
            int i;
            for(i=0; i<MAX_PLAYERS; i++)
            {
                if(g_pPlayers[i])
                {
                    g_pPlayers[i]->MarkToBeDeleted();
                    g_pPlayers[i] = NULL;
                }
            }
            g_PlayerBeingControlled = -1;
            g_pObjectManager->DeleteDeadObjects();
            g_pObjectManager->DeleteAllOfType(OBJECTTYPE_PowerUp);
            g_pObjectManager->DeleteAllOfType(OBJECTTYPE_Flame);

            MessageGameInfo* mess = (MessageGameInfo*)message;
            if(GameClient)
            {
                g_pGameBoard->Clear(1);
                g_pGameBoard->SetupFromPacket(mess);
                g_FrameCount = mess->framecount;
                g_RoundStarted = mess->roundstarted;
            }
        }
        break;

    case NM_AddPlayer:
        {
            MessageAddPlayer* mess = (MessageAddPlayer*)message;

            if(GameClient)
                break;

            if(g_RoundStarted)
                break;

            int i;

            //MessageAddPlayer* mess = (MessageAddPlayer*)message;

            //Check if there's already a player from this machine.
            for(i=0; i<MAX_PLAYERS; i++)
                if(g_pPlayers[i])
                    if(g_pPlayers[i]->m_MachineNum == conn)
                        return;

            // Find an empty spot in the players array.
            for(i=0; i<MAX_PLAYERS; i++)
                if(g_pPlayers[i] == NULL)
                    break;

            // Tell everyone(including the server) that someone joined.
            if(i<MAX_PLAYERS)
            {
                MessagePlayerJoined* messtosend = MyNew MessagePlayerJoined();
                messtosend->player = i;
                messtosend->machinewhocontrolsit = conn;
                strcpy(messtosend->name, mess->name);
                messtosend->score = g_PlayerWins[conn+1];
                messtosend->Send();
                SAFE_REDUCEREFANDDELETE(messtosend);
            }
        }
        break;

    case NM_PlayerJoined:
        {
            MessagePlayerJoined* mess = (MessagePlayerJoined*)message;

            g_pPlayers[mess->player] = (PlayerObject*)g_pObjectManager->AddPlayer();
            if(g_pPlayers[message->player] == NULL)
                break;

            strcpy(g_PlayerName[mess->machinewhocontrolsit+1], mess->name);
            g_PlayerWins[mess->machinewhocontrolsit+1] = mess->score;

            g_pPlayers[mess->player]->m_MachineNum = mess->machinewhocontrolsit;
            if(g_MachineNumber == mess->machinewhocontrolsit)
                g_PlayerBeingControlled = mess->player;

            int pos = mess->player;

            if(pos < 4)
                g_pPlayers[mess->player]->SetPosZ(LEVEL2_Z);

            if(pos > 3)
            {
                //g_pPlayers[mess->player]->SetPosZ(LEVEL2_Z);
                pos -= 4;
            }


            if(pos == 0)
                g_pPlayers[mess->player]->MoveAbs(0, 0);
            if(pos == 1)
                g_pPlayers[mess->player]->MoveAbs(TILE_WIDTH*(MAP_WIDTH-1), TILE_HEIGHT*(MAP_HEIGHT-1));
            if(pos == 2)
                g_pPlayers[mess->player]->MoveAbs(0, TILE_HEIGHT*(MAP_HEIGHT-1));
            if(pos == 3)
                g_pPlayers[mess->player]->MoveAbs(TILE_WIDTH*(MAP_WIDTH-1), 0);
        }
        break;

    case NM_PlayerLeft:
        if(g_pPlayers[message->player])
            g_pPlayers[message->player]->MarkToBeDeleted();
        break;

    case NM_StartGame:
        g_RoundStarted = 1;
        break;

    case NM_RoundEnded:
        g_RoundStarted = 0;
        g_WaitingForGameState = 1;
        break;

    case NM_SetOnFire:
        if(g_pPlayers[message->player])
            g_pPlayers[message->player]->SetOnFire();            
        break;

    case NM_PlayerMove:
        {
            MessagePlayerMove* mess = (MessagePlayerMove*)message;

            if(mess->player == g_PlayerBeingControlled)
                break;

            if(g_pPlayers[mess->player])
            {
                g_pPlayers[mess->player]->MoveAbs(mess->x, mess->y);
                g_pPlayers[mess->player]->SetPosZ(mess->z);
                g_pPlayers[mess->player]->SetKeysHeld( mess->keys );
            }
        }
        break;

    case NM_RequestBomb:
        {
            if(GameClient)
                break;

            MessageRequestBomb* mess = (MessageRequestBomb*)message;

            ObjectType type = g_pGameBoard->GetTileObjectType(mess->map, mess->tx, mess->ty);
            if(type == OBJECTTYPE_None || type == OBJECTTYPE_FakeBomb)
            {
                MessageDropBomb* messtosend = MyNew(MessageDropBomb);
                messtosend->player = mess->player;
                messtosend->map = mess->map;
                messtosend->tx = mess->tx;
                messtosend->ty = mess->ty;
                messtosend->range = mess->range;
                messtosend->TickToExplode = g_FrameCount + TOTALTIMETOEXPLODE;
                messtosend->Send();
                SAFE_REDUCEREFANDDELETE(messtosend);
            }
        }
        break;

    case NM_DropBomb:
        {
            MessageDropBomb* mess = (MessageDropBomb*)message;

            // find the "fake" bomb from that square.
            BombObject* obj = NULL;
            ObjectType type = g_pGameBoard->GetTileObjectType(mess->map, mess->tx, mess->ty);
            if(type == OBJECTTYPE_FakeBomb)
            {
                int index = g_pGameBoard->GetTileIndex(mess->map, mess->tx, mess->ty);
                obj = (BombObject*)g_pObjectManager->GetObject(type, index);
                if(obj)
                    g_pGameBoard->SetTile(OBJECTTYPE_Bomb, index, mess->map, mess->tx, mess->ty);
                else
                    obj = (BombObject*)g_pObjectManager->AddBomb(mess->tx<<4, mess->ty<<4, mess->map?LEVEL2_Z:0);
            }
            else            
                obj = (BombObject*)g_pObjectManager->AddBomb(mess->tx<<4, mess->ty<<4, mess->map?LEVEL2_Z:0);

            if(obj)
            {
                obj->SetPlayer(mess->player);
                obj->SetWaitingForActivation(0);
                obj->SetRange(mess->range);
                obj->SetTickToExplode(mess->TickToExplode);
            }
        }
        break;

    case NM_MoveElevator:
        {
            MessageMoveElevator* mess = (MessageMoveElevator*)message;

            if(mess->ElevatorGroup == 0)
            {
                g_pGameBoard->m_Elevators[0]->TouchElevator(mess->TickToMove);
                g_pGameBoard->m_Elevators[1]->TouchElevator(mess->TickToMove);
            }
            else if(mess->ElevatorGroup == 1)
            {
                g_pGameBoard->m_Elevators[2]->TouchElevator(mess->TickToMove);
                g_pGameBoard->m_Elevators[3]->TouchElevator(mess->TickToMove);
            }
        }
        break;

    case NM_GrabPowerUp:
        {
            MessageGrabPowerUp* mess = (MessageGrabPowerUp*)message;

            int type = g_pGameBoard->GetTileObjectType(mess->map, mess->tx, mess->ty);

            if(type == OBJECTTYPE_PowerUp)
            {
                PowerUpObject* obj = (PowerUpObject*)g_pGameBoard->GetTileObject(mess->map, mess->tx, mess->ty);
                int power = obj->QueryPowerType();

                if(power == POWERTYPE_Bomb)
                {
                    PlaySoundOnNextAvailableChannel(GAMESOUND_ExtraBomb);
                    g_pPlayers[mess->player]->m_NumBombs++;
                }
                else if(power == POWERTYPE_Gas)
                {
                    PlaySoundOnNextAvailableChannel(GAMESOUND_ExtraGas);
                    g_pPlayers[mess->player]->m_Power++;
                }

                g_pGameBoard->ClearTile(1, mess->map, mess->tx, mess->ty);
            }
            //if(mess->ElevatorGroup == 0)
            //{
            //    g_pGameBoard->m_Elevators[0]->TouchElevator(mess->TickToMove);
            //    g_pGameBoard->m_Elevators[1]->TouchElevator(mess->TickToMove);
            //}
            //else if(mess->ElevatorGroup == 1)
            //{
            //    g_pGameBoard->m_Elevators[2]->TouchElevator(mess->TickToMove);
            //    g_pGameBoard->m_Elevators[3]->TouchElevator(mess->TickToMove);
            //}
        }
        break;

    case NM_BombFlood:
        {
            MessageBombFlood* mess = (MessageBombFlood*)message;

            int x,y;

            for(x=0; x<MAP_WIDTH; x++)
            {
                for(y=0; y<MAP_HEIGHT; y++)
                {
                    if( ( mess->side == 0 && (x == 0 || x == MAP_WIDTH-1) ) ||
                        ( mess->side == 1 && (y == 0 || y == MAP_HEIGHT-1) ) )
                    {
                        BombObject* obj = NULL;
                        ObjectType type = g_pGameBoard->GetTileObjectType(mess->map, x, y);
                        if(type == OBJECTTYPE_None)
                            obj = (BombObject*)g_pObjectManager->AddBomb(x<<4, y<<4, mess->map?LEVEL2_Z:0);

                        if(obj)
                        {
                            obj->SetPlayer(-1);
                            obj->SetWaitingForActivation(0);
                            obj->SetRange(50);
                            obj->SetTickToExplode(g_FrameCount + 60*4);
                        }
                    }
                }
            }
        }
        break;

    case NM_PlayerInfo:
        {
            MessagePlayerInfo* mess = (MessagePlayerInfo*)message;

            int i;
            for(i=0; i<MAX_PLAYERS; i++)
            {
                if(i == g_PlayerBeingControlled)
                    continue;

                if(g_pPlayers[i])
                {
                    g_pPlayers[i]->MoveAbs(mess->GetX(i), mess->GetY(i));
                    g_pPlayers[i]->SetPosZ(mess->GetZ(i));
                    g_pPlayers[i]->SetKeysHeld( mess->GetKeys(i) );
                }
            }
        }
        break;

    default:
        break;
    }
}
