#include "MySystem.h"

#include "BaseObject.h"
#include "BombGraphics.h"

BombObject::BombObject()
{
    m_Type = OBJECTTYPE_Bomb;
    m_Timer = TOTALTIMETOEXPLODE;
    m_TicksWithNoHealth = 0;
    m_SwappedGraphics = 0;
    m_DidDamage = 0;
    m_CurrentAnim = BOMBANIM_Idle;
}

BombObject::~BombObject()
{
}

void BombObject::SetRange(int range)
{
    m_Range = range;
}

void BombObject::SetTickToExplode(int tick)
{
    m_Timer = tick - g_FrameCount;
    if(m_Timer < 0)
        m_Timer = 1;
}

void BombObject::Tick()
{
    if(m_WaitingForActivation)
        return;

    m_Timer--;

    if(m_Health >= 0)
    {
        if(m_Timer < TOTALTIMETOEXPLODE/3)
            m_CurrentAnim = BOMBANIM_Spark3;
        else if(m_Timer < (TOTALTIMETOEXPLODE/3)*2)
            m_CurrentAnim = BOMBANIM_Spark2;
        else
            m_CurrentAnim = BOMBANIM_Spark1;
    }

    if(m_Timer <= 0)
        m_Health = -1;

    if(m_Health < 0)
    {
        int tx = (int)m_PosX / 16;
        int ty = (int)m_PosY / 16;

        int map = QueryMap((int)m_PosZ);
        //if(map == -1)
        //    return;

        m_Health = -1;

        m_TicksWithNoHealth++;
        
        if(!m_SwappedGraphics)
        {
            PlaySoundOnNextAvailableChannel(GAMESOUND_Explosion);
            m_SwappedGraphics = 1;
            SetTileInfo(&BombExplodeTI, 0, 1);
            m_CurrentAnim = 0;
            m_Frame = 0;
            m_Delay = 0;
        }
        else if(m_Frame == 8 && m_Delay >= m_AnimSpeed-1)
        {
            m_ToBeDeleted = 1;

            g_pGameBoard->ClearTile(0, map, tx, ty);
        }
    }

    if(!m_DidDamage && m_TicksWithNoHealth > 3)
    {
        m_DidDamage = 1;

        int tx = (int)m_PosX / 16;
        int ty = (int)m_PosY / 16;

        int map = QueryMap((int)m_PosZ);
        if(map == -1)
            return;

        ObjectType type;

        type = g_pGameBoard->GetTileObjectType(map, tx, ty);
        if(type == OBJECTTYPE_BreakableBlock || type == OBJECTTYPE_Bomb || type == OBJECTTYPE_PowerUp)
        {
            BaseObject* obj = g_pGameBoard->GetTileObject(map, tx, ty);
            obj->AddHealth(-1001);
        }

        if(tx < MAP_WIDTH-1)
        {
            type = g_pGameBoard->GetTileObjectType(map, tx+1, ty);
            if(type == OBJECTTYPE_BreakableBlock || type == OBJECTTYPE_Bomb || type == OBJECTTYPE_PowerUp)
            {
                BaseObject* obj = g_pGameBoard->GetTileObject(map, tx+1, ty);
                obj->AddHealth(-1001);
                //g_pGameBoard->ClearTile(0, map, tx+1, ty);
            }
            else if(type != OBJECTTYPE_Block && type != OBJECTTYPE_Flame)
            {
                FlameObject* obj = (FlameObject*)g_pObjectManager->AddFlame((tx+1)*16, ty*16, (int)m_PosZ);
                obj->SetLife(m_Range);
                obj->SetCurrentAnim(FlameANIM_Right);
            }
        }

        if(tx > 0)
        {
            type = g_pGameBoard->GetTileObjectType(map, tx-1, ty);
            if(type == OBJECTTYPE_BreakableBlock || type == OBJECTTYPE_Bomb || type == OBJECTTYPE_PowerUp)
            {
                BaseObject* obj = g_pGameBoard->GetTileObject(map, tx-1, ty);
                obj->AddHealth(-1001);
                //g_pGameBoard->ClearTile(0, map, tx-1, ty);
            }
            else if(type != OBJECTTYPE_Block && type != OBJECTTYPE_Flame)
            {
                FlameObject* obj = (FlameObject*)g_pObjectManager->AddFlame((tx-1)*16, ty*16, (int)m_PosZ);
                obj->SetLife(m_Range);
                obj->SetCurrentAnim(FlameANIM_Right);
                obj->SetFlipHorizontal(1);
            }
        }

        if(ty < MAP_HEIGHT-1)
        {
            type = g_pGameBoard->GetTileObjectType(map, tx, ty+1);
            if(type == OBJECTTYPE_BreakableBlock || type == OBJECTTYPE_Bomb || type == OBJECTTYPE_PowerUp)
            {
                BaseObject* obj = g_pGameBoard->GetTileObject(map, tx, ty+1);
                obj->AddHealth(-1001);
                //g_pGameBoard->ClearTile(0, map, tx, ty+1);
            }
            else if(type != OBJECTTYPE_Block && type != OBJECTTYPE_Flame)
            {
                FlameObject* obj = (FlameObject*)g_pObjectManager->AddFlame(tx*16, (ty+1)*16, (int)m_PosZ);
                obj->SetLife(m_Range);
                obj->SetCurrentAnim(FlameANIM_Down);
            }
        }

        if(ty > 0)
        {
            type = g_pGameBoard->GetTileObjectType(map, tx, ty-1);
            if(type == OBJECTTYPE_BreakableBlock || type == OBJECTTYPE_Bomb || type == OBJECTTYPE_PowerUp)
            {
                BaseObject* obj = g_pGameBoard->GetTileObject(map, tx, ty-1);
                obj->AddHealth(-1001);
                //g_pGameBoard->ClearTile(0, map, tx, ty-1);
            }
            else if(type != OBJECTTYPE_Block && type != OBJECTTYPE_Flame)
            {
                FlameObject* obj = (FlameObject*)g_pObjectManager->AddFlame(tx*16, (ty-1)*16, (int)m_PosZ);
                obj->SetLife(m_Range);
                obj->SetCurrentAnim(FlameANIM_Up);
            }
        }
    }
}
