#include "MySystem.h"

int QueryMap(int z)
{
    if(z == 0)
        return 0;
    else if(z == LEVEL2_Z)
        return 1;

    return -1;
}

GameBoard::GameBoard()
{
    m_Elevators[0] = (ElevatorObject*)g_pObjectManager->AddElevator( 4*TILE_WIDTH,
                                                                     2*TILE_HEIGHT+4,
                                                                     LEVEL2_Z);
    m_Elevators[0]->SetElevatorNumber(0);

    m_Elevators[1] = (ElevatorObject*)g_pObjectManager->AddElevator( 4*TILE_WIDTH,
                                                                     6*TILE_HEIGHT+4,
                                                                     0);
    m_Elevators[1]->SetElevatorNumber(2);

    m_Elevators[2] = (ElevatorObject*)g_pObjectManager->AddElevator( 10*TILE_WIDTH,
                                                                     2*TILE_HEIGHT+4,
                                                                     0);
    m_Elevators[2]->SetElevatorNumber(1);

    m_Elevators[3] = (ElevatorObject*)g_pObjectManager->AddElevator( 10*TILE_WIDTH,
                                                                     6*TILE_HEIGHT+4,
                                                                     LEVEL2_Z);
    m_Elevators[3]->SetElevatorNumber(3);

    Clear(0);
}

GameBoard::~GameBoard()
{
}

BaseObject* GameBoard::GetTileObject(int map, int x, int y)
{
    if(x < 0 || x >= MAP_WIDTH)
        return NULL;
    if(y < 0 || y >= MAP_HEIGHT)
        return NULL;

    return g_pObjectManager->GetObject(m_Tiles[map][x][y].type, m_Tiles[map][x][y].ObjectManagerIndex);
}

ObjectType GameBoard::GetTileObjectType(int map, int x, int y)
{
    if(x < 0 || x >= MAP_WIDTH)
        return OBJECTTYPE_NumTypes;
    if(y < 0 || y >= MAP_HEIGHT)
        return OBJECTTYPE_NumTypes;

    return m_Tiles[map][x][y].type;
}

int GameBoard::GetTileIndex(int map, int x, int y)
{
    if(x < 0 || x >= MAP_WIDTH)
        return OBJECTTYPE_NumTypes;
    if(y < 0 || y >= MAP_HEIGHT)
        return OBJECTTYPE_NumTypes;

    return m_Tiles[map][x][y].ObjectManagerIndex;
}

void GameBoard::SetTile(ObjectType type, int index, int map, int x, int y)
{
    m_Tiles[map][x][y].type = type;
    m_Tiles[map][x][y].ObjectManagerIndex = index;
}

void GameBoard::ClearTile(int deleteobjects, int map, int x, int y)
{
    if(deleteobjects)
        g_pObjectManager->RemoveObject(m_Tiles[map][x][y].type, m_Tiles[map][x][y].ObjectManagerIndex);

    m_Tiles[map][x][y].type = OBJECTTYPE_None;
}

void GameBoard::Clear(int deleteobjects)
{
    int map, x, y;

    for(map=0; map<NUM_MAPS; map++)
        for(x=0; x<MAP_WIDTH; x++)
            for(y=0; y<MAP_HEIGHT; y++)
                ClearTile(deleteobjects, map, x, y);

    m_Elevators[0]->SetPosZ(LEVEL2_Z);
    m_Elevators[1]->SetPosZ(0);
    m_Elevators[2]->SetPosZ(0);
    m_Elevators[3]->SetPosZ(LEVEL2_Z);
}

void GameBoard::Setup()
{
    int map, x, y;

    Clear(1);

    //return;

    for(map=0; map<NUM_MAPS; map++)
    {
        for(x=0; x<MAP_WIDTH; x++)
        {
            for(y=0; y<MAP_HEIGHT; y++)
            {
                if(x%2 == 1 && y%2 == 1)
                {
                    m_Tiles[map][x][y].type = OBJECTTYPE_Block;

                    g_pObjectManager->AddBlock( x*TILE_WIDTH,
                                                y*TILE_HEIGHT,
                                                map*LEVEL2_Z );
                }
                else //if(x%2 != 1 || y%2 != 1)
                {
                    if(y==0)
                        if(x==0 || x==1 || x==MAP_WIDTH-1 || x==MAP_WIDTH-2)
                            continue;
                    if(y==1)
                        if(x==0 || x==MAP_WIDTH-1)
                            continue;
                    if(y==MAP_HEIGHT-1)
                        if(x==0 || x==1 || x==MAP_WIDTH-1 || x==MAP_WIDTH-2)
                            continue;
                    if(y==MAP_HEIGHT-2)
                        if(x==0 || x==MAP_WIDTH-1)
                            continue;

                    if( (x == 4 && y == 2) ||
                        (x == 4 && y == 6) ||
                        (x == 10 && y == 2) ||
                        (x == 10 && y == 6) )
                        continue;

                    if(rand()%2 == 0)
                        continue;

                    m_Tiles[map][x][y].type = OBJECTTYPE_BreakableBlock;

                    int pow = 0;
                    int r = rand()%3;
                    if(r == 0)
                        pow = rand()%2 + 1;

                    g_pObjectManager->AddBreakableBlock( x*TILE_WIDTH,
                                                         y*TILE_HEIGHT,
                                                         map*LEVEL2_Z, pow );
                }
            }
        }
    }
}

void GameBoard::SetupFromPacket(MessageGameInfo* mess)
{
    int map, x, y;

    m_Elevators[0]->SetPosZ(LEVEL2_Z);
    m_Elevators[1]->SetPosZ(0);
    m_Elevators[2]->SetPosZ(0);
    m_Elevators[3]->SetPosZ(LEVEL2_Z);

    Clear(1);

    for(map=0; map<NUM_MAPS; map++)
    {
        for(x=0; x<MAP_WIDTH; x++)
        {
            for(y=0; y<MAP_HEIGHT; y++)
            {
                int block = mess->blocks[map][x][y] & 0xF;
                int pow = mess->blocks[map][x][y] >> 4;

                if(block == OBJECTTYPE_Block)
                {
                    m_Tiles[map][x][y].type = OBJECTTYPE_Block;

                    g_pObjectManager->AddBlock( x*TILE_WIDTH,
                                                y*TILE_HEIGHT,
                                                map*LEVEL2_Z );
                }
                else if(block == OBJECTTYPE_BreakableBlock)
                {
                    m_Tiles[map][x][y].type = OBJECTTYPE_BreakableBlock;

                    g_pObjectManager->AddBreakableBlock( x*TILE_WIDTH,
                                                         y*TILE_HEIGHT,
                                                         map*LEVEL2_Z, pow );
                }
            }
        }
    }
}

void GameBoard::SendBoardLayout(int client)
{
    int map, x, y;

    MessageGameInfo* mess = MyNew(MessageGameInfo);

    for(map=0; map<NUM_MAPS; map++)
    {
        for(x=0; x<MAP_WIDTH; x++)
        {
            for(y=0; y<MAP_HEIGHT; y++)
            {
                mess->blocks[map][x][y] = (u8)m_Tiles[map][x][y].type;
                if(m_Tiles[map][x][y].type == OBJECTTYPE_BreakableBlock)
                {
                    BreakableBlockObject* obj = (BreakableBlockObject*)GetTileObject(map, x, y);
                    mess->blocks[map][x][y] |= obj->QueryPowerUp() << 4;
                }
            }
        }
    }
    
    mess->framecount = g_FrameCount;
    mess->roundstarted = g_RoundStarted;
    mess->Send(client);
    SAFE_REDUCEREFANDDELETE(mess);
}


