#include "MySystem.h"
#include "ObjectManager.h"
#include "BombGraphics.h"

ObjectManager::ObjectManager()
{
    int i;

    for(i=0; i<MAX_PLAYERS; i++)
        m_Players[i] = NULL;
    
    for(i=0; i<MAX_BLOCKS; i++)
    {
        m_Blocks[i] = NULL;
        m_BreakableBlocks[i] = NULL;
        m_Bombs[i] = NULL;
        m_Flames[i] = NULL;
        m_PowerUps[i] = NULL;
    }

    for(i=0; i<4; i++)
        m_Elevators[i] = NULL;
}

ObjectManager::~ObjectManager()
{
    int i;

    for(i=0; i<MAX_PLAYERS; i++)
        SAFE_DELETE(m_Players[i]);

    for(i=0; i<MAX_BLOCKS; i++)
    {
        SAFE_DELETE(m_Blocks[i]);
        SAFE_DELETE(m_BreakableBlocks[i]);
        SAFE_DELETE(m_Bombs[i]);
        SAFE_DELETE(m_Flames[i]);
        SAFE_DELETE(m_PowerUps[i]);
    }

    for(i=0; i<4; i++)
        SAFE_DELETE(m_Elevators[i]);
}

void ObjectManager::TickAllObjects()
{
    int i;

    int numticked = 0;

    for(i=0; i<4; i++)
    {
        if(m_Elevators[i])
        {
            m_Elevators[i]->Tick();
            numticked++;
        }
    }

    for(i=0; i<MAX_PLAYERS; i++)
    {
        if(m_Players[i])
        {
            m_Players[i]->Tick();
            numticked++;
        }
    }

    for(i=0; i<MAX_BLOCKS; i++)
    {
        if(m_Bombs[i])
        {
            m_Bombs[i]->Tick();
            numticked++;
        }

        if(m_BreakableBlocks[i])
        {
            m_BreakableBlocks[i]->Tick();
            numticked++;
        }

        if(m_Flames[i])
        {
            m_Flames[i]->Tick();
            numticked++;
        }

        if(m_PowerUps[i])
        {
            m_PowerUps[i]->Tick();
            numticked++;
        }
    }
}

void ObjectManager::UpdateObjectSprites()
{
    int i;

    int numupdated1 = 0;
    int numupdated2 = 0;
    int numupdated3 = 0;
    int numupdated4 = 0;
    int numupdated5 = 0;
    int numupdated6 = 0;
    int numupdated7 = 0;

    for(i=0; i<MAX_PLAYERS; i++)
    {
        if(m_Players[i])
        {
            m_Players[i]->UpdateSprite();
            numupdated1++;
        }
    }

    for(i=0; i<MAX_BLOCKS; i++)
    {
        if(m_Blocks[i])
        {
            m_Blocks[i]->UpdateSprite();
            numupdated2++;
        }
        if(m_BreakableBlocks[i])
        {
            m_BreakableBlocks[i]->UpdateSprite();
            numupdated3++;
        }
        if(m_Bombs[i])
        {
            m_Bombs[i]->UpdateSprite();
            numupdated4++;
        }
        if(m_Flames[i])
        {
            m_Flames[i]->UpdateSprite();
            numupdated5++;
        }
        if(m_PowerUps[i])
        {
            m_PowerUps[i]->UpdateSprite();
            numupdated6++;
        }
    }

    for(i=0; i<4; i++)
    {
        if(m_Elevators[i])
        {
            m_Elevators[i]->UpdateSprite();
            numupdated7++;
        }
    }

    sprintf(DEBUG_String, "numupdated:%02d,%02d,%02d,%02d,%02d,%02d,%02d,",
            numupdated1, numupdated2, numupdated3, numupdated4, numupdated5, numupdated6, numupdated7 );
    DEBUG_Print();
}

BaseObject* ObjectManager::AddPlayer()
{
    int i;
    for(i=0; i<MAX_PLAYERS; i++)
        if(m_Players[i] == NULL)
            break;

    if(i >= MAX_PLAYERS)
        return NULL;

    m_Players[i] = MyNew(PlayerObject);
    m_Players[i]->SetTileInfo(&PlayerTI, 0);
    ((PlayerObject*)m_Players[i])->SetPlayerNum(i);

    return m_Players[i];
}

BaseObject* ObjectManager::AddBlock(int x, int y, int z)
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
        if(m_Blocks[i] == NULL)
            break;

    if(i >= MAX_BLOCKS)
        return NULL;

    int map = QueryMap(z);
    if(map == -1)
        return NULL;

    g_pGameBoard->SetTile(OBJECTTYPE_Block, i, map, x>>4, y>>4);

    m_Blocks[i] = MyNew(BlockObject);
    m_Blocks[i]->MoveAbs((float)x, (float)y);
    m_Blocks[i]->SetPosZ((float)z);
    int start = 64;
    int inc = 25;
    if(x < start)
    {
        m_Blocks[i]->SetTileInfo(&BlockTI, 0);
    }
    else if(x < start+inc)
    {
        m_Blocks[i]->SetTileInfo(&Block2TI, 0);
    }
    else if(x < start+inc*3)
    {
        m_Blocks[i]->SetTileInfo(&Block3TI, 0);
    }
    else if(x < start+inc*4)
    {
        m_Blocks[i]->SetFlipHorizontal(1);
        m_Blocks[i]->SetTileInfo(&Block2TI, 0);
    }
    else
    {
        m_Blocks[i]->SetFlipHorizontal(1);
        m_Blocks[i]->SetTileInfo(&BlockTI, 0);
    }

    return m_Blocks[i];
}

BaseObject* ObjectManager::AddBreakableBlock(int x, int y, int z, int powerup)
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
        if(m_BreakableBlocks[i] == NULL)
            break;

    if(i >= MAX_BLOCKS)
        return NULL;

    int map = QueryMap(z);
    if(map == -1)
        return NULL;

    g_pGameBoard->SetTile(OBJECTTYPE_BreakableBlock, i, map, x>>4, y>>4);

    m_BreakableBlocks[i] = MyNew(BreakableBlockObject);
    m_BreakableBlocks[i]->MoveAbs((float)x, (float)y);
    m_BreakableBlocks[i]->SetPosZ((float)z);
    ((BreakableBlockObject*)m_BreakableBlocks[i])->SetPowerUp(powerup);
    if(z == 0)
        m_BreakableBlocks[i]->SetTileInfo(&BreakableBlockTI, 0);
    else
        m_BreakableBlocks[i]->SetTileInfo(&BreakableBlock2TI, 0);

    return m_BreakableBlocks[i];
}

BaseObject* ObjectManager::AddBomb(int x, int y, int z, int fake)
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
        if(m_Bombs[i] == NULL)
            break;

    if(i >= MAX_BLOCKS)
        return NULL;

    int map = QueryMap(z);
    if(map == -1)
        return NULL;

    ObjectType type = g_pGameBoard->GetTileObjectType(map, x>>4, y>>4);

    if(type != OBJECTTYPE_None)
        return NULL;

    if(fake)
        g_pGameBoard->SetTile(OBJECTTYPE_FakeBomb, i, map, x>>4, y>>4);
    else        
        g_pGameBoard->SetTile(OBJECTTYPE_Bomb, i, map, x>>4, y>>4);

    int px = x; 
    px = ((px>>4)<<4);
    int py = y;
    py = ((py>>4)<<4);

    m_Bombs[i] = MyNew(BombObject);
    m_Bombs[i]->MoveAbs((float)px, (float)py);
    m_Bombs[i]->SetPosZ((float)z);
    m_Bombs[i]->SetTileInfo(&BombTI, 0, 1);
    if(fake)
    {
        ((BombObject*)m_Bombs[i])->SetWaitingForActivation(1);
    }
    PlaySoundOnNextAvailableChannel(GAMESOUND_BombDrop);

    return m_Bombs[i];
}

BaseObject* ObjectManager::AddFlame(int x, int y, int z)
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
        if(m_Flames[i] == NULL)
            break;

    if(i >= MAX_BLOCKS)
        return NULL;

    int map = QueryMap(z);
    if(map == -1)
        return NULL;

    ObjectType type = g_pGameBoard->GetTileObjectType(map, x>>4, y>>4);

    if(type != OBJECTTYPE_None)
        return NULL;

    g_pGameBoard->SetTile(OBJECTTYPE_Flame, i, map, x>>4, y>>4);

    int px = x; 
    px = ((px>>4)<<4);
    int py = y;
    py = ((py>>4)<<4);

    m_Flames[i] = MyNew(FlameObject);
    m_Flames[i]->MoveAbs((float)px, (float)py);
    m_Flames[i]->SetPosZ((float)z);
    m_Flames[i]->SetTileInfo(&FlamesTI, 0, 1);

    return m_Flames[i];
}

BaseObject* ObjectManager::AddPowerUp(int x, int y, int z, int powertype)
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
        if(m_PowerUps[i] == NULL)
            break;

    if(i >= MAX_BLOCKS)
        return NULL;

    int map = QueryMap(z);
    if(map == -1)
        return NULL;

    ObjectType type = g_pGameBoard->GetTileObjectType(map, x>>4, y>>4);

    if(type != OBJECTTYPE_None)
        return NULL;

    g_pGameBoard->SetTile(OBJECTTYPE_PowerUp, i, map, x>>4, y>>4);

    int px = x; 
    px = ((px>>4)<<4);
    int py = y;
    py = ((py>>4)<<4);

    m_PowerUps[i] = MyNew(PowerUpObject);
    m_PowerUps[i]->MoveAbs((float)px, (float)py);
    m_PowerUps[i]->SetPosZ((float)z);
    ((PowerUpObject*)m_PowerUps[i])->SetPowerType(powertype);
    if(powertype == POWERTYPE_Bomb)
        m_PowerUps[i]->SetTileInfo(&PowerUpBombTI, 0, 1);
    else
        m_PowerUps[i]->SetTileInfo(&PowerUpGasTI, 0, 1);

    return m_PowerUps[i];
}

BaseObject* ObjectManager::AddElevator(int x, int y, int z)
{
    int i;
    for(i=0; i<4; i++)
        if(m_Elevators[i] == NULL)
            break;

    if(i >= 4)
        return NULL;

    m_Elevators[i] = MyNew(ElevatorObject);
    m_Elevators[i]->MoveAbs((float)x, (float)y);
    m_Elevators[i]->SetPosZ((float)z);
    m_Elevators[i]->SetTileInfo(&ElevatorTI, 0, 1);

    return m_Elevators[i];
}

void ObjectManager::RemoveObject(ObjectType type, int index)
{
    if(type == OBJECTTYPE_Player)
    {
        SAFE_DELETE(m_Players[index]);
    }
    if(type == OBJECTTYPE_Block)
    {
        SAFE_DELETE(m_Blocks[index]);
    }
    if(type == OBJECTTYPE_BreakableBlock)
    {
        SAFE_DELETE(m_BreakableBlocks[index]);
    }
    else if(type == OBJECTTYPE_Bomb)
    {
        SAFE_DELETE(m_Bombs[index]);
    }
    else if(type == OBJECTTYPE_Flame)
    {
        SAFE_DELETE(m_Flames[index]);
    }
    else if(type == OBJECTTYPE_PowerUp)
    {
        SAFE_DELETE(m_PowerUps[index]);
    }
    else if(type == OBJECTTYPE_Elevator)
    {
        SAFE_DELETE(m_Elevators[index]);
    }
}

void ObjectManager::DeleteAllOfType(ObjectType type)
{
    int i;

    if(type == OBJECTTYPE_BreakableBlock)
    {
        for(i=0; i<MAX_BLOCKS; i++)
            SAFE_DELETE(m_BreakableBlocks[i]);
    }
    else if(type == OBJECTTYPE_Bomb)
    {
        for(i=0; i<MAX_BLOCKS; i++)
            SAFE_DELETE(m_Bombs[i]);
    }
    else if(type == OBJECTTYPE_Flame)
    {
        for(i=0; i<MAX_BLOCKS; i++)
            SAFE_DELETE(m_Flames[i]);
    }
    else if(type == OBJECTTYPE_PowerUp)
    {
        for(i=0; i<MAX_BLOCKS; i++)
            SAFE_DELETE(m_PowerUps[i]);
    }
}

BaseObject* ObjectManager::GetObject(ObjectType type, int index)
{
    if(type == OBJECTTYPE_BreakableBlock)
        return m_BreakableBlocks[index];
    else if(type == OBJECTTYPE_Bomb || type == OBJECTTYPE_FakeBomb)
        return m_Bombs[index];
    else if(type == OBJECTTYPE_Flame)
        return m_Flames[index];
    else if(type == OBJECTTYPE_PowerUp)
        return m_PowerUps[index];

    return NULL;
}

void ObjectManager::DeleteDeadObjects()
{
    int i;
    for(i=0; i<MAX_BLOCKS; i++)
    {
        if(m_Players[i])
            if(m_Players[i]->QueryToBeDeleted())
                SAFE_DELETE(m_Players[i]);

        if(m_BreakableBlocks[i])
            if(m_BreakableBlocks[i]->QueryToBeDeleted())
                SAFE_DELETE(m_BreakableBlocks[i]);

        if(m_Bombs[i])
            if(m_Bombs[i]->QueryToBeDeleted())
                SAFE_DELETE(m_Bombs[i]);

        if(m_Flames[i])
            if(m_Flames[i]->QueryToBeDeleted())
                SAFE_DELETE(m_Flames[i]);

        if(m_PowerUps[i])
            if(m_PowerUps[i]->QueryToBeDeleted())
                SAFE_DELETE(m_PowerUps[i]);
    }

    for(i=0; i<4; i++)
    {
        if(m_Elevators[i])
            if(m_Elevators[i]->QueryToBeDeleted())
                SAFE_DELETE(m_Elevators[i]);
    }
}

int ObjectManager::CountBombsByPlayer(int player)
{
    int i;
    int numbombs = 0;

    for(i=0; i<MAX_BLOCKS; i++)
    {
        if(m_Bombs[i])
        {
            if( ((BombObject*)m_Bombs[i])->GetPlayer() == player )
                if(m_Bombs[i]->QueryHealth() > 0)
                    numbombs++;
        }
    }

    return numbombs;
}
