#include "MySystem.h"

#include "BaseObject.h"
#include "MyDMA.h"

#include "BitmapDraw.h"
#include "SpriteRamHandler.h"
#include "ObjectManager.h"

#include "BombGraphics.h"

#include "WifiMenu.h"

#define TIMETOLIVE  (60*5)

PlayerObject::PlayerObject() :
    m_DirectionFaced(OBJECTDIR_Down),    
    m_PlayerNum(0)
{
    m_Type = OBJECTTYPE_Player;
    m_WalkSpeed = 1.0f;
    m_AnimSpeed = 5;
    m_MachineNum = -2;
    m_MovementLocked = 0;
    m_TimeToLive = 0;
    m_KeysHeld = 0;
    m_Power = 1;
    m_NumBombs = 1;
}

PlayerObject::~PlayerObject()
{
}

void PlayerObject::SetOnFire()
{
    m_Health = -1;
    m_TimeToLive = TIMETOLIVE;
    m_MovementLocked = 1;
    m_AnimSpeed = 5;
    m_CurrentAnim = PLAYERANIM_CaughtFire;
    m_Frame = m_TI->AnimFrameOffset[m_CurrentAnim];
    m_Delay = 0;
    PlaySoundOnNextAvailableChannel(GAMESOUND_Death01);
}

void PlayerObject::Tick()
{
    int i;

    if(m_Health <= 0)
        m_Health = -1;

    int map = QueryMap((int)m_PosZ);
    int ntx = ((int)m_PosX+8) >> 4;
    int nty = ((int)m_PosY+8) >> 4;

    ObjectType nto = OBJECTTYPE_None;

    if(m_CurrentAnim == PLAYERANIM_Win)
    {
        int lastframeofanim = m_TI->AnimFrameOffset[m_CurrentAnim] + m_TI->numAnimFrames[m_CurrentAnim] - 1;

        if(m_Frame == lastframeofanim && m_Delay >= m_AnimSpeed-1)
        {
            m_AnimSpeed++;
            if(m_AnimSpeed > 7)
                m_AnimSpeed = 7;
            m_Frame -= 3;
            m_Delay = 0;
        }
    }
    else if(g_RoundStarted && m_CurrentAnim != PLAYERANIM_Died && (GameServer || GameClient))
    {
        // Check if you're the last player alive and not on fire, and play the win anim.
        int numplayers = 0;
        for(i=0; i<MAX_PLAYERS; i++)
        {
            if(g_pPlayers[i])
            {
                int anim = g_pPlayers[i]->QueryCurrentAnim();
                if(!g_pPlayers[i]->QueryToBeDeleted() &&
                    anim != PLAYERANIM_Died)
                    numplayers++;
            }
        }

        if(numplayers == 1 && 
            m_CurrentAnim != PLAYERANIM_CaughtFire && m_CurrentAnim != PLAYERANIM_RunOnFire )
        {
            m_AnimSpeed = 5;
            m_MovementLocked = 1;
            m_CurrentAnim = PLAYERANIM_Win;
            m_Frame = m_TI->AnimFrameOffset[m_CurrentAnim];
            m_Delay = 0;
        }
    }

    if(map != -1)
    {
        nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);

        // if we're standing in a flame, kill ourself.
        // only do this check for local players.
        if(m_Health > 0 && m_MachineNum == g_MachineNumber)
        {
            int standinginfire = 0;

            int i;
            for(i=0; i<MAX_PLAYERS; i++)
            {
                if(g_pPlayers[i])
                {
                    if(g_pPlayers[i]->QueryCurrentAnim() != PLAYERANIM_RunOnFire)
                        continue;

                    float diffx = m_PosX - g_pPlayers[i]->QueryPosX();
                    float diffy = m_PosY - g_pPlayers[i]->QueryPosY();
                    float diffz = m_PosZ - g_pPlayers[i]->QueryPosZ();

                    //TODO: don't check if you're touching yourself this way.
                    if(diffx != 0 || diffy != 0 || diffz != 0)
                    {
                        int dx = abs((int)diffx);
                        int dy = abs((int)diffy);
                        int dz = abs((int)diffz);

                        if(dx < 8 && dy < 8 && dz < 8)
                        {
                            standinginfire = 1;
                        }
                    }
                }
            }

            if(nto == OBJECTTYPE_Flame)
            {
                standinginfire = 1;
            }

            if(nto == OBJECTTYPE_Bomb)
            {
                BombObject* obj = (BombObject*)g_pGameBoard->GetTileObject(map, ntx, nty);
                if(obj->QueryTileInfo() == &BombExplodeTI)
                    standinginfire = 1;
            }

            if(standinginfire)
            {
                MessageSetOnFire* mess = MyNew(MessageSetOnFire);
                mess->player = g_PlayerBeingControlled;
                mess->Send();
                SAFE_REDUCEREFANDDELETE(mess);
            }
        }

        if(m_Health < 1)
        {
            if(m_TimeToLive > 0)
            {
                // if we run screaming into a flame, relight on fire.
                // you have to be on fire for a second before you can recatch fire.
                if(!m_MovementLocked && nto == OBJECTTYPE_Flame && m_TimeToLive < TIMETOLIVE - 60)
                {
                    SetOnFire();
                }
                else
                {
                    int lastframeofanim = m_TI->AnimFrameOffset[m_CurrentAnim] + m_TI->numAnimFrames[m_CurrentAnim] - 1;

                    // if on fire.
                    if(!m_MovementLocked)
                        m_TimeToLive--;

                    // if done catching fire.
                    if(m_MovementLocked && m_Frame == lastframeofanim && m_Delay >= m_AnimSpeed-1)
                    {
                        m_MovementLocked = 0;
                        m_WalkSpeed = 1.25f;
                        m_AnimSpeed = 8;
                        m_CurrentAnim = PLAYERANIM_RunOnFire;
                        m_Frame = m_TI->AnimFrameOffset[m_CurrentAnim];
                        m_Delay = 0;
                    }
                }
            }
            else
            {
                if(!m_MovementLocked)
                {
                    m_MovementLocked = 1;
                    m_AnimSpeed = 5;
                    m_CurrentAnim = PLAYERANIM_Died;
                    m_Frame = m_TI->AnimFrameOffset[m_CurrentAnim];
                    m_Delay = 0;
                }
                else if(m_MachineNum == g_MachineNumber && m_Frame == m_TI->numAnimFrames[m_CurrentAnim] && m_Delay >= m_AnimSpeed-1)
                {
                    MessagePlayerLeft* mess = MyNew(MessagePlayerLeft);
                    mess->player = g_PlayerBeingControlled;
                    mess->Send();
                    SAFE_REDUCEREFANDDELETE(mess);
                }
            }
        }
        else if(m_CurrentAnim != PLAYERANIM_Win)
        {
            if(m_LastX != m_PosX || m_LastY != m_PosY)
            {
                if(m_CurrentAnim != PLAYERANIM_Walk)
                    m_Delay = m_AnimSpeed;
                m_CurrentAnim = PLAYERANIM_Walk;
            }
            else
            {
                //if(m_CurrentAnim != OBJECTANIM_Idle)
                //    m_Delay = m_AnimSpeed;
                m_CurrentAnim = PLAYERANIM_Idle;
            }
        }
    }

    if(!GameClient) // only the host can trigger powerups pickups.
    {        
        if(g_pGameBoard->GetTileObjectType(map, ntx, nty) == OBJECTTYPE_PowerUp)
        {
            MessageGrabPowerUp* mess = MyNew(MessageGrabPowerUp);
            mess->map = map;
            mess->tx = ntx;
            mess->ty = nty;
            mess->player = m_PlayerNum;
            mess->Send();
            SAFE_REDUCEREFANDDELETE(mess);
        }
    }

    // override the sort order only if the player is on an elevator.
    m_OverrideSortPos = 9999; //(int)m_PosY - 32

    // if the player in on an elevator square, do some special case stuff.
    int elevator = -1;
    if(ntx == 4 && nty == 2)
        elevator = 0;
    if(ntx == 4 && nty == 6)
        elevator = 1;
    if(ntx == 10 && nty == 2)
        elevator = 2;
    if(ntx == 10 && nty == 6)
        elevator = 3;

    if(elevator != -1)
    {
        int z = g_pGameBoard->m_Elevators[elevator]->QueryPosZ();

        if(!GameClient) // only the host can trigger the elevators.
        {
            if(m_PosZ == z)
            {
                if( g_pGameBoard->m_Elevators[elevator]->IsElevatorReadyToMove() )
                {
                    MessageMoveElevator* mess = MyNew(MessageMoveElevator);
                    if(elevator == 0 || elevator == 1)
                        mess->ElevatorGroup = 0;
                    else
                        mess->ElevatorGroup = 1;
                    mess->TickToMove = g_FrameCount + ELEVATORDELAY;
                    mess->Send();
                    SAFE_REDUCEREFANDDELETE(mess);
                }
            }
        }

        // if you're above the elevator, drop to it.
        if(m_PosZ > z)
        {
            m_PosZ -= 10;
            if(m_PosZ < z)
                m_PosZ = (float)z;

            if(m_PosZ != 0 && m_PosZ != LEVEL2_Z)
            {
                m_PosX = (float)(ntx<<4);
                m_PosY = (float)(nty<<4);
                if(m_PosZ > 192-32)
                    m_OverrideSortPos = (int)m_PosY - 27;
            }
        }

        if( abs((int)m_PosZ - z) < 10 )
        {
            m_PosZ = (float)z;

            if(m_PosZ != 0 && m_PosZ != LEVEL2_Z)
            {
                m_PosX = (float)(ntx<<4);
                m_PosY = (float)(nty<<4);
                if(m_PosZ > 192-32)
                    m_OverrideSortPos = (int)m_PosY - 27;
            }
        }

        // if you're up to 10 pixels lower than the elevator, clamp to it.
        //if(m_PosZ - z < 10)
        //{
        //    m_PosZ += 10;
        //    if(m_PosZ > LEVEL2_Z)
        //        m_PosZ = LEVEL2_Z;
        //}

    }

    BaseObject::Tick();
}

void PlayerObject::SendMovePacket(int playernum)
{
    MessagePlayerMove* mess = MyNew(MessagePlayerMove);
    mess->player = playernum;
    mess->x = m_PosX;
    mess->y = m_PosY;
    mess->z = m_PosZ;
    mess->keys = m_KeysHeld;
    mess->Send();
    SAFE_REDUCEREFANDDELETE(mess);
}

void PlayerObject::SetKeysHeld(int keys)
{
    m_KeysHeld = keys;
}

void PlayerObject::ProcessKeys()
{
    if(m_MovementLocked)
        return;

    float dx = 0;
    float dy = 0;

    float diff = m_WalkSpeed;

    if(m_KeysHeld & KEY_UP)
        dy = -diff;
    if(m_KeysHeld & KEY_DOWN)
        dy = diff;
    if(m_KeysHeld & KEY_LEFT)
        dx = -diff;
    if(m_KeysHeld & KEY_RIGHT)
        dx = diff;

    //if( !OnlineMatch || g_RoundStarted )
    // only check for bomb drops on local players... and use the ds keys for that.
    if(m_MachineNum == g_MachineNumber)
    {
        if(!m_MovementLocked && (m_CurrentAnim == PLAYERANIM_Walk || m_CurrentAnim == PLAYERANIM_Idle))
        {
            if( keysUp() & KEY_L || keysUp() & KEY_R ||
                keysUp() & KEY_A || keysUp() & KEY_B ||
                keysUp() & KEY_X || keysUp() & KEY_Y )
            {
                int numbombs = g_pObjectManager->CountBombsByPlayer(g_PlayerBeingControlled);
                if(numbombs < m_NumBombs)
                {
                    int map = QueryMap((int)m_PosZ);
                    if(map != -1)
                    {
                        int tx = ((int)m_PosX+8)>>4;
                        int ty = ((int)m_PosY+8)>>4;

                        if( (tx == 4 && ty == 2) ||
                            (tx == 4 && ty == 6) ||
                            (tx == 10 && ty == 2) ||
                            (tx == 10 && ty == 6) )
                        {
                            // no bombs on elevators, no time to debug :)
                        }
                        else
                        {
                            ObjectType type = g_pGameBoard->GetTileObjectType(map, tx, ty);
                            if(type == OBJECTTYPE_None)
                            {
                                // place a fake bomb while waiting for the server to place a real one.
                                BombObject* obj = (BombObject*)g_pObjectManager->AddBomb((int)m_PosX+8, (int)m_PosY+8, (int)m_PosZ, 1);
                                if(obj)
                                {
                                    obj->SetPlayer(g_PlayerBeingControlled);

                                    MessageRequestBomb* mess = MyNew(MessageRequestBomb);
                                    mess->player = g_PlayerBeingControlled;
                                    mess->tx = ((int)m_PosX+8)>>4;
                                    mess->ty = ((int)m_PosY+8)>>4;
                                    mess->map = map;
                                    mess->range = m_Power;
                                    //mess->TickToExplode = 0;
                                    mess->Send();
                                    SAFE_REDUCEREFANDDELETE(mess);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    MoveRelative(dx, dy);
}

void PlayerObject::MoveAbs(float x, float y)
{
    BaseObject::MoveAbs(x,y);
}

int PlayerObject::CheckIfObjectFits(int map, int x, int y)
{
    int ntx, nty;
    ObjectType nto;

    ntx = (x) >> 4;
    nty = (y) >> 4;
    nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);
    if(nto != OBJECTTYPE_None && nto != OBJECTTYPE_Bomb && nto != OBJECTTYPE_FakeBomb && nto != OBJECTTYPE_Flame && nto != OBJECTTYPE_PowerUp)
        return 0;

    ntx = (x+15) >> 4;
    nty = (y+15) >> 4;
    nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);
    if(nto != OBJECTTYPE_None && nto != OBJECTTYPE_Bomb && nto != OBJECTTYPE_FakeBomb && nto != OBJECTTYPE_Flame && nto != OBJECTTYPE_PowerUp)
        return 0;

    ntx = (x+15) >> 4;
    nty = (y) >> 4;
    nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);
    if(nto != OBJECTTYPE_None && nto != OBJECTTYPE_Bomb && nto != OBJECTTYPE_FakeBomb && nto != OBJECTTYPE_Flame && nto != OBJECTTYPE_PowerUp)
        return 0;

    ntx = (x) >> 4;
    nty = (y+15) >> 4;
    nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);
    if(nto != OBJECTTYPE_None && nto != OBJECTTYPE_Bomb && nto != OBJECTTYPE_FakeBomb && nto != OBJECTTYPE_Flame && nto != OBJECTTYPE_PowerUp)
        return 0;

    return 1;
}

void PlayerObject::MoveRelative(float x, float y)
{
    m_LastX = m_PosX;
    m_LastY = m_PosY;

    if(x == 0 && y == 0)
        return;

    int map = QueryMap((int)m_PosZ);
    if(map == -1)
        return;

    float posx = m_PosX;
    float posy = m_PosY;

    if(posx + x < 0)
        x = 0;
    if(posx + x + 15 >= MAP_WIDTH*TILE_WIDTH)
        x = 0;
    if(posy + y < 0)
        y = 0;
    if(posy + y + 15 >= MAP_HEIGHT*TILE_HEIGHT)
        y = 0;

    // can you reach your destination
    if(!CheckIfObjectFits(map, (int)(posx+x), (int)(posy+y)))
    {
        // can you just to sideways
        if(x == 0 || !CheckIfObjectFits(map, (int)(posx+x), (int)(posy)))
        {
            // can you just to vertically.
            if(y == 0 || !CheckIfObjectFits(map, (int)(posx), (int)(posy+y)))
            {
                // can't go anywhere so...
                // if you pressing sideways, move vertically.
                // This is really hardcoded for an evenly spaced grid.
                int slack = 10;
                if(x)
                {
                    x = 0;
                    int dy = ((int)m_PosY) % 32;
                    if(dy <= 1+slack && dy >= 1)
                        y = -1;
                    else if(dy <= 31 && dy >= 31-slack)
                        y = 1;
                    if(!CheckIfObjectFits(map, (int)(posx+x), (int)(posy+y)))
                    {
                        return;
                    }
                }
                else //if(y) // if you pressing up/down, move sideways.
                {
                    y = 0;
                    int dx = ((int)m_PosX) % 32;
                    int slack = 12;
                    if(dx <= 1+slack && dx >= 1)
                        x = -1;
                    else if(dx <= 31 && dx >= 31-slack)
                        x = 1;
                    if(!CheckIfObjectFits(map, (int)(posx+x), (int)(posy+y)))
                    {
                        return;
                    }
                }
            }
            else
                x = 0;
        }
        else
            y = 0;
    }

    int ntx = ((int)m_PosX+8) >> 4;
    int nty = ((int)m_PosY+8) >> 4;
    // if we're moving to a new tile, check if it's a bomb.
    if(((int)(posx+x)+8) >> 4 != ntx || ((int)(posy+y)+8) >> 4 != nty)
    {
        if(x > 0)
            ntx++;
        if(x < 0)
            ntx--;
        if(y > 0)
            nty++;
        if(y < 0)
            nty--;
        ObjectType nto = g_pGameBoard->GetTileObjectType(map, ntx, nty);
        if(nto == OBJECTTYPE_Bomb || nto == OBJECTTYPE_FakeBomb)
            return;
    }

    m_PosX = posx+x;
    m_PosY = posy+y;

    return;
}

int PlayerObject::QueryDirectionFaced()
{
    float diffx, diffy;

    diffx = m_LastX - m_PosX;
    diffy = m_LastY - m_PosY;

    if(fabs(diffx) > fabs(diffy))
    {
        if(diffx < 0)
            return OBJECTDIR_Right;
        else
            return OBJECTDIR_Left;
    }
    else
    {
        if(diffy > 0)
            return OBJECTDIR_Up;
        else
            return OBJECTDIR_Down;
    }
}
