#include "MySystem.h"

#include <malloc.h>

#include "touch.h"
#include "string.h"
#include "TextMap.h"
#if SYSTEM_NDS
#include "FIFO.h"
#endif //SYSTEM_NDS

#include "WifiMenu.h"
#include "ServerList.h"
#include "Menu.h"

#include "NetMessages.h"
#include "NetCode.h"

#include "GameLobby.h"
#include "ServerSelect.h"

NetClient* GameClient = NULL;
NetServer* GameServer = NULL;

int WIFIconnected = 0;
unsigned long serverip;

#if SYSTEM_NDS

extern "C"
{
    // Dswifi stub functions
    void* sgIP_malloc(int size)
    {
        return MyNew(char[size]);
    }

    void sgIP_free(void* ptr)
    {
        delete[] (char*)ptr;
    }

    // sgIP_dbgprint only needed in debug version
    void sgIP_dbgprint(char* txt, ...)
    {
    }
}

#define     VCOUNT      (*((u16 volatile *) 0x04000006))

// wifi timer function, to update internals of sgIP
void Timer_50ms(void)
{
   Wifi_Timer(50);
}

// notification function to send fifo message to arm7
void arm9_synctoarm7() // send fifo message
{
    FIFO_Message* message = FIFO_GetMessageContainer();
    if(message)
    {
        message->command = MESSAGE_WIFI_SYNC;
        FIFO_SendMessage(message);
    }
}

#endif //SYSTEM_NDS

int WIFIMain(void)
{
    SERVERLIST_ClearList();

    static int doneonce = 0;

    if(!doneonce)
    {
        doneonce = 1;

        u32 Wifi_pass = Wifi_Init(WIFIINIT_OPTION_USELED);
        FIFO_Message* message = FIFO_GetMessageContainer();
        if(message)
        {
            message->command = MESSAGE_WIFI_INIT;
            message->len = Wifi_pass;
            FIFO_SendMessage(message);
        }

        irqSet(IRQ_TIMER3, Timer_50ms); // setup timer IRQ
        irqEnable(IRQ_TIMER3);
        Wifi_SetSyncHandler(arm9_synctoarm7); // tell wifi lib to use our handler to notify arm7

        // set timer3
        *((volatile u16 *)0x0400010C) = -6553; // 6553.1 * 256 cycles = ~50ms;
        *((volatile u16 *)0x0400010E) = 0x00C2; // enable, irq, 1/256 clock

        while(Wifi_CheckInit() == 0) // wait for arm7 to be initted successfully
        {
            //swiWaitForVBlank();
            while(VCOUNT>192); // wait for vblank
            while(VCOUNT<192);
        }
    }

    Wifi_EnableWifi();

    if(1)
    { // simple WFC connect:
        int i;
        Wifi_AutoConnect(); // request connect
        int anim = 0;
        int animspeed = 0;
        while(1)
        {
            animspeed++;

            if(animspeed > 200)
            {
                animspeed = 0;
                anim++;
                if(anim >= 3)
                    anim = 0;
            }

            else if(anim == 0)
                DrawTextToMap(1, NULL, 10, 6, TextColor, "Connecting /");
            else if(anim == 1)
                DrawTextToMap(1, NULL, 10, 6, TextColor, "Connecting -");
            else if(anim == 2)
                DrawTextToMap(1, NULL, 10, 6, TextColor, "Connecting \\");

            i=Wifi_AssocStatus(); // check status
            if(i == ASSOCSTATUS_ASSOCIATED)
            {
                //iprintf("Connected successfully!\n");
                WIFIconnected = 1;
                break;
            }
            if(i == ASSOCSTATUS_CANNOTCONNECT)
            {
                //iprintf("Could not connect!\n");
                break;
            }
        }
    } // if connected, you can now use the berkley sockets interface to connect to the internet!

    if(WIFIconnected != 1)
        return 0;

    if( SERVERLIST_Init() )
        return 0;

    serverip = Wifi_GetIP(); //0x6501a8c0;
    //serverip = 0x65010000;
    int currshift = 4*6;

    int SL_Returncode = 0;

    int keeplooping = 1;
    int currentselected = 0;

    while(keeplooping)
    {
        // menu
        scanKeys();

        swiWaitForVBlank();

        // clear the screen
        ClearTextMap(NULL);

        if(!GameServer && !GameClient)
        {
            DrawTextToMap(1, NULL, 11, 3, TextColor, "WIFI Menu");

            unsigned long serverip = Wifi_GetIP();

            sprintf(temptext, "%d.%d.%d.%d",
                (int)serverip&0xFF, ((int)serverip>>8)&0xFF, ((int)serverip>>16)&0xFF, ((int)serverip>>24)&0xFF );
            DrawTextToMap(1, NULL, 2, 19, TextColor, "Note: To host an online game,");
            DrawTextToMap(1, NULL, 2, 20, TextColor, "you need forward port 9999 to");
            DrawTextToMap(1, NULL, 2, 21, TextColor, " this ip:"); 
            DrawTextToMap(1, NULL, 12, 21, TextColor, temptext);

#define LineStatus  20

            //sprintf(temptext, "LAN IP Hack");
            //DrawTextToMap(1, NULL, 15, LineStatus, TextColor, temptext);

            //sprintf(temptext, "ip) %08lx", serverip);
            //DrawTextToMap(1, NULL, 15, LineStatus+1, TextColor, temptext);

            //sprintf(temptext, "^");
            //DrawTextToMap(1, NULL, 18+8-currshift/4, LineStatus+2, TextColor, temptext);

#define LineList     7
#define LineListLan  11
#define LineAdd      15

            DrawTextToMap(1, NULL, 9, LineList, TextColor,    "  Join Game");
            DrawTextToMap(1, NULL, 9, LineListLan, TextColor, "Join Lan Game");
            DrawTextToMap(1, NULL, 9, LineAdd, TextColor,     "  Host Game");
        }
        static int TouchX;
        static int TouchY;

        if( keysUp() & KEY_UP )
            currentselected--;
        if( keysUp() & KEY_DOWN )
            currentselected++;

        if(currentselected < 0)
            currentselected = 0;
        if(currentselected > 3-1)
            currentselected = 3-1;

        DrawTextToMap(1, NULL, 5, LineList+currentselected*4, TextColor, "->");
        
        if(keysHeld() & KEY_TOUCH)
        {
            static int FrameCount = 0;
            FrameCount++;
            GetTouchXY(FrameCount, &TouchX, &TouchY);
        }

        //if( keysUp() & KEY_LEFT )
        //{
        //    currshift += 4;
        //    if(currshift > 28)
        //        currshift = 28;
        //}
        //if( keysUp() & KEY_RIGHT )
        //{
        //    currshift -= 4;
        //    if(currshift < 0)
        //        currshift = 0;
        //}
        //if( keysUp() & KEY_UP )
        //{
        //    serverip += 1 << currshift;
        //}
        //if( keysUp() & KEY_DOWN )
        //{
        //    serverip -= 1 << currshift;
        //}

        int StartServer = 0;
        int StartClient = 0;
        int StartClientLan = 0;

        if((keysUp() & KEY_A || keysUp() & KEY_START) && currentselected == 0)
            StartClient = 1;
        if((keysUp() & KEY_A || keysUp() & KEY_START) && currentselected == 1)
            StartClientLan = 1;
        if((keysUp() & KEY_A || keysUp() & KEY_START) && currentselected == 2)
            StartServer = 1;

        if(keysUp() & KEY_TOUCH)
        {
            SL_Returncode = 0;
            if(TouchY >= LineList*8 - 4 && TouchY < LineList*8 + 12)
                if(TouchX >= 14*8 && TouchX < 32*8)
                    StartClient = 1;

            if(TouchY >= LineListLan*8 - 4 && TouchY < LineListLan*8 + 12)
                if(TouchX >= 14*8 && TouchX < 32*8)
                    StartClientLan = 1;

            if(TouchY >= LineAdd*8 - 4 && TouchY < LineAdd*8 + 12)
                if(TouchX >= 14*8 && TouchX < 32*8)
                    StartServer = 1;
        }

        //if( keysUp() & KEY_A )
        //    StartServer = 1;

        //if( keysUp() & KEY_Y )
        //    StartClient = 1;

        if( StartServer )
        {
            if( Menu_AddHostToServerList() == 0 )
            {
                int ret = GameLobbyMain(1); // start a server

                SERVERLIST_RemoveServer();

                if(ret)
                    keeplooping = 0;
            }
            continue;
        }

        if( StartClient || StartClientLan )
        {
            if( Menu_ServerSelect(StartClientLan) == 0 )
            {
                int ret = GameLobbyMain(0); // start a client

                if(ret)
                    keeplooping = 0;
            }

            continue;
        }

        //if( keysUp() & KEY_R )
        //{
        //    Menu_ServerSelect(1);
        //}

        //// direct connect to server
        //if( keysUp() & KEY_X )
        //{
        //    int ret = GameLobbyMain(0);

        //    if(ret)
        //        keeplooping = 0;

        //    continue;
        //}

        if( keysUp() & KEY_B )
        {
            keeplooping = 0;
            continue;
        }
    }

#if SYSTEM_NDS
    Wifi_DisconnectAP();
    Wifi_DisableWifi();
#endif //SYSTEM_NDS

    SAFE_DELETE(GameServer);
    SAFE_DELETE(GameClient);

    return 0;
}

int Menu_AddHostToServerList()
{
    int timeposted = g_FrameCount;

    // clear the screen
    ClearTextMap(NULL);

    SERVERLIST_AddServer();

    while(1)
    {
        scanKeys();
#if SYSTEM_NDS
        swiWaitForVBlank();
#endif //SYSTEM_NDS

        DrawTextToMap(1, NULL, 8, 4, TextColor, "Posting Server");
        DrawTextToMap(1, NULL, 5, 6, TextColor, "Press 'B' to cancel");

        if(g_FrameCount > timeposted + 120)
        {
            timeposted = g_FrameCount;
            SERVERLIST_AddServer();
        }

        int SL_Returncode = SERVERLIST_CheckForResults();

        if(SL_Returncode != SLRC_Waiting)
        {
            return 0; // server is now listed
        }

        if( keysUp() & KEY_B )
        {
            SERVERLIST_CancelOp();
            SERVERLIST_RemoveServer();
            return 1; // error
        }
   }

    return 1; // error
}
