#include "MySystem.h"

#include "TextMap.h"
#include "ServerList.h"

#include "Menu.h"
#include "GameLobby.h"

#include "BombMessages.h"
#include "BombGraphics.h"

#define SERVER_PORT     9999
#define MAXPENDING      5

uint16* Map0_Sub;

int PrintToScreenCurLine = 0;

void PrintToScreenReset()
{
    PrintToScreenCurLine = 0;
}

void PrintToScreen(char* text, int linein, int debugonly)
{
    if(debugonly && _RELEASE)
        return;

    int x;
    Map0_Sub = (uint16*)SCREEN_BASE_BLOCK_SUB(FONT_SCREEN_BASE_BLOCK);

    int lineout = PrintToScreenCurLine;
    if(linein != -1)
        lineout = linein;
    
    for(x=0; x<32; x++) Map0_Sub[lineout*32+x] = FONT_TILE_BLANK;
    DrawTextToMap(1, NULL, 0, lineout, TextColor, text);

    if(linein == -1)
    {
        PrintToScreenCurLine++;
        if(PrintToScreenCurLine >= 20)
            PrintToScreenCurLine = 0;
    }
}

NetClient::NetClient() :
    m_ServerSock(0),
    m_ServerPort(0),
    m_TickLastMessageSent(0),
    m_TickLastMessageRecvd(0)
{
    memset(&m_ServerAddr, 0, sizeof(sockaddr_in));
    m_PacketStack = MyNew(PacketStack);
}

NetClient::~NetClient()
{
    Close();
    SAFE_DELETE(m_PacketStack);
}

void NetClient::Init(int port)
{
    m_ServerSock = socket(AF_INET, SOCK_DGRAM, 0);
    m_ServerPort = port;

    // set non-blocking
    int i=1;
    ioctl(m_ServerSock, FIONBIO, &i);

    sprintf(temptext, "ServerSock - %d", m_ServerSock);
    PrintToScreen(temptext);
}

void NetClient::Connect(unsigned long ip)
{
    in_addr ServerAddr;
    ServerAddr.s_addr = ip; //0x6501a8c0; //ip;

    memset( &m_ServerAddr, 0, sizeof(sockaddr_in) );
    m_ServerAddr.sin_family = AF_INET;
    m_ServerAddr.sin_addr = ServerAddr;
    m_ServerAddr.sin_port = htons(m_ServerPort);

    sprintf(temptext, "Connecting to - %lx", ip);
    PrintToScreen(temptext, 0, 0);
    sprintf(temptext, "Press 'B' to cancel", ip);
    PrintToScreen(temptext, 1, 0);

    m_PacketStack->SetConnectionInfo(0, m_ServerSock, &m_ServerAddr);
}

void NetClient::Send(NetMessage* message, int size)
{
    if( m_ServerAddr.sin_port != 0 )
    {
        m_TickLastMessageSent = g_FrameCount;
        //int ret = 
        m_PacketStack->SendTo(message->reliable, m_ServerSock, message, size, 0, (sockaddr*)&m_ServerAddr, sizeof(sockaddr_in));

        sprintf(temptext, "send - %d, %d", message->id, message->type);
        PrintToScreen(temptext);
    }
    //else
    //{
    //    SAFE_DELETE(message);
    //}
}

void NetClient::Close()
{
    close(m_ServerSock);
    m_PacketStack->Cleanup();
}

int NetClient::Tick()
{
    m_PacketStack->Tick(m_ServerSock);

    char buffer[MAX_MESSAGE_SIZE];
    sockaddr_in sain;
    int sizeofsaddr = sizeof(sockaddr_in);
    int numbytes = 1;

    // if we haven't sent the server a message in a half second, send a keep alive.
    // otherwise the server might think we turned off.
    if(g_FrameCount - m_TickLastMessageSent > 30)
    {
        m_TickLastMessageSent = g_FrameCount;
        m_KeepAlive.Send(0); // send to the server only, don't bounce it back to us... not that it matters.
    }

    while(1)
    {
        numbytes = recvfrom(m_ServerSock, buffer, MAX_MESSAGE_SIZE, 0, (sockaddr*)&sain, &sizeofsaddr);

        if(numbytes == 0)
        {
            // Socket is closed... this shouldn't happen
            int bp = 1;
            bp = 1;
        }

        if(numbytes == -1)
            break;

        m_PacketStack->ProcessMessage(m_ServerSock, buffer, numbytes, 0);
        m_TickLastMessageRecvd = g_FrameCount;
    }

    // if we haven't received a message from the server in 3 seconds, assume we lost connection.
    if(g_FrameCount - m_TickLastMessageRecvd > NETWORK_TIMEOUT)
    {
        return 1;
    }

    return 0;
}

int NetClient::ProcessMessage(NetMessage* message, int conn)
{
    switch(message->type)
    {
    case NM_Welcome:
        //GameOptions.InitialSeed = ((MessageStartGame*)message)->seed;
        //SAFE_DELETE(message);
        g_HackStartGame = 1;
        g_MachineNumber = ((MessageWelcome*)message)->machinenumber;
        //BombMain();
        //return 1;
        break;

    default:
        //g_LockStepQueue.AddMessage(message);
        ProcessGameMessages(message, conn);
        break;
    }

    //SAFE_DELETE(message);

    static int nummessages = 0;
    //sprintf(temptext, "Message#%d, id)%d, type)%d, %d", nummessages, message->id, message->type, message->tick);
    //PrintToScreen(temptext);
    nummessages++;

    return 0;
}

int NetClient::IsThisTheServer(sockaddr_in* sain)
{
    if(sain->sin_addr.s_addr == m_ServerAddr.sin_addr.s_addr && sain->sin_port == m_ServerAddr.sin_port)
        return 1;

    return 0;
}

NetServer::NetServer() :
    m_ServerSock(0),
    m_Port(0),
    m_NumClients(0),
    m_Joinable(0)
{
    int i;

    memset( &m_ServerAddr, 0, sizeof(sockaddr_in) );
    for(i=0; i<MAX_CLIENTS; i++)
    {
        //m_ClientSock[i] = 0;
        memset( &m_ClientAddr[i], 0, sizeof(sockaddr_in) );
        m_TickLastMessageSent[i] = 0;
        m_TickLastMessageRecvd[i] = 0;
        m_PacketStack[i] = MyNew PacketStack;
    }
}

NetServer::~NetServer()
{
    int i;
    ValidateAllocations();
    Close();
    for(i=0; i<MAX_CLIENTS; i++)
        SAFE_DELETE(m_PacketStack[i])

    i = 0;
}

void NetServer::Init(int port)
{
    m_Port = port;

    int ret;

    unsigned long ip = 0;

#if SYSTEM_NDS
    ip = Wifi_GetIP();
#endif

    sprintf(temptext, "Server Initialized");
    PrintToScreen(temptext, -1);
    sprintf(temptext, "Waiting for Client");
    PrintToScreen(temptext, -1);

    sprintf(temptext, "IP - %lx", ip);
    PrintToScreen(temptext, -1);

    // Create socket for incoming connections
    m_ServerSock = socket(PF_INET, SOCK_DGRAM, 0); //IPPROTO_TCP)

    sprintf(temptext, "Socket - %d", m_ServerSock);
    PrintToScreen(temptext);

    // Construct local address structure
    memset(&m_ServerAddr, 0, sizeof(sockaddr_in));      // Zero out structure
    m_ServerAddr.sin_family = AF_INET;                  // Internet address family
    m_ServerAddr.sin_addr.s_addr = htonl(INADDR_ANY);   // Any incoming interface
    m_ServerAddr.sin_port = htons(m_Port);              // Local port

    // Bind to the local address
    ret = bind( m_ServerSock, (sockaddr*)&m_ServerAddr, sizeof(sockaddr_in) );

    sprintf(temptext, "Bind - %d", ret);
    PrintToScreen(temptext);

    //// Mark the socket so it will listen for incoming connections
    //ret = listen( m_ServerSock, MAXPENDING );

    //sprintf(temptext, "Listen - %d", ret);
    //PrintToScreen(temptext);

    int i=1;
    ioctl(m_ServerSock, FIONBIO, &i);

    m_Joinable = 1;
}

void NetServer::CheckForIncomingConnections()
{
    //int i;
    //for(i=0; i<MAX_CLIENTS; i++)
    //{
    //    if(m_ClientSock[i] == 0)
    //        break;
    //}

    //if(i == MAX_CLIENTS)
    //    return; // error

    //int clntLen = sizeof(sockaddr_in);
    //int ret = accept( m_ServerSock, (sockaddr*)&m_ClientAddr[i], &clntLen );
    //if(ret != -1)
    //{
    //    m_ClientSock[i] = ret;

    //    sprintf(temptext, "accept - %d - %d", i, ret);
    //    PrintToScreen(temptext);
    //}
}

void NetServer::Send(NetMessage* message, int size, int clienttoignore, int clienttosendto)
{
    int i;
    for(i=0; i<MAX_CLIENTS; i++)
    {
        if(clienttoignore == i)
            continue;

        if(clienttosendto != -1 && clienttosendto != i)
            continue;

        if( m_ClientAddr[i].sin_port != 0 )
        {
            m_TickLastMessageSent[i] = g_FrameCount;
            //int ret = 
            m_PacketStack[i]->SendTo(message->reliable, m_ServerSock, message, size, 0, (sockaddr*)&m_ClientAddr[i], sizeof(sockaddr_in));

            sprintf(temptext, "send - %d, %d", message->id, message->type);
            PrintToScreen(temptext);
        }
    }
}

void NetServer::Close()
{
    int i;
    
    SERVERLIST_RemoveServer();

    for(i=0; i<MAX_CLIENTS; i++)
    {
        memset(&m_ClientAddr[i], 0, sizeof(sockaddr_in));
        m_PacketStack[i]->Cleanup();
        //close(m_ClientSock[i]);
    }

    m_NumClients = 0;

    close(m_ServerSock);
}

int NetServer::AddClient(sockaddr_in* sain, int CheckIfNewClient)
{
    int i;

    // find out if the client was already connected
    for(i=0; i<MAX_CLIENTS; i++)
    {
        if( m_ClientAddr[i].sin_port != 0 )
        {
            //sprintf(temptext, "New-%lx,%d,%d,%s", sain.sin_addr.s_addr, sain.sin_family, sain.sin_port, sain.sin_zero);
            //PrintToScreen(temptext);
            //sprintf(temptext, "Old-%lx,%d,%d,%s", m_ClientAddr[i].sin_addr.s_addr, m_ClientAddr[i].sin_family, m_ClientAddr[i].sin_port, m_ClientAddr[i].sin_zero);
            //PrintToScreen(temptext);

            //if( !memcmp(&m_ClientAddr[i], &sain, sizeofsaddr) )
            if(sain->sin_addr.s_addr == m_ClientAddr[i].sin_addr.s_addr && sain->sin_port == m_ClientAddr[i].sin_port)
            {
                return i;
            }
        }
    }

    if(CheckIfNewClient == 0)
        return -1;

    for(i=0; i<MAX_CLIENTS; i++)
        if( m_ClientAddr[i].sin_port == 0 )
            break;

    if(i == MAX_CLIENTS)
    {
        // TODO: handle this? not really needed
        sprintf(temptext, "too many clients");
        PrintToScreen(temptext);
        return -1;
    }

    //m_Joinable = 0;
    m_NumClients++;

    m_Joinable = 1;
    if(m_NumClients == MAX_CLIENTS)
        m_Joinable = 0;

#if SYSTEM_NDS
    // ugly hack to update server, close, update, then recreate the socket.
    {
        close(m_ServerSock);

        SERVERLIST_UpdateServer(m_Joinable, m_NumClients);

        m_ServerSock = socket(PF_INET, SOCK_DGRAM, 0);
        bind( m_ServerSock, (sockaddr*)&m_ServerAddr, sizeof(sockaddr_in) );
        int i=1;
        ioctl(m_ServerSock, FIONBIO, &i);
    }
#endif //SYSTEM_NDS

    m_ClientAddr[i] = *sain;
    m_PacketStack[i]->SetConnectionInfo(i, m_ServerSock, &m_ClientAddr[i]);

    MessageWelcome* mess = MyNew(MessageWelcome);
    mess->machinenumber = i;
    mess->Send(i);
    SAFE_REDUCEREFANDDELETE(mess);

    sprintf(temptext, "new client - %d - %lx", i, m_ClientAddr[i].sin_addr.s_addr);
    PrintToScreen(temptext, -1);

    sprintf(temptext, "Press 'A' to start game");
    PrintToScreen(temptext, -1);

    return i;
}

int NetServer::IsClientConnected(int conn)
{
    if(m_ClientAddr[conn].sin_port != 0)
        return 1;

    return 0;
}

void NetServer::RemoveClient(int conn)
{
    m_PacketStack[conn]->Cleanup();
    memset( &m_ClientAddr[conn], 0, sizeof(sockaddr_in) );
    m_TickLastMessageRecvd[conn] = 0;
}

int NetServer::Tick()
{
    int i;
    for(i=0; i<MAX_CLIENTS; i++)
    {
        if( m_ClientAddr[i].sin_port != 0 )
            m_PacketStack[i]->Tick(m_ServerSock);
    }

    char buffer[MAX_MESSAGE_SIZE];
    sockaddr_in sain;
    int sizeofsaddr = sizeof(sockaddr_in);
    int numbytes = 1;

    while(1)
    {
        numbytes = recvfrom(m_ServerSock, buffer, MAX_MESSAGE_SIZE, 0, (sockaddr*)&sain, &sizeofsaddr);

        if(numbytes == 0)
        {
            // Socket is closed... this shouldn't happen
            int bp = 1;
            bp = 1;
        }

        if(numbytes == -1)
            break;

        int PossibleNewClient = 0;
        if(((NetMessage*)buffer)->type == NM_JoinServer)
            PossibleNewClient = 1;

        int conn = AddClient(&sain, PossibleNewClient);
        if(conn != -1)
        {
            m_PacketStack[conn]->ProcessMessage(m_ServerSock, buffer, numbytes, conn);
            m_TickLastMessageRecvd[conn] = g_FrameCount;
        }
    }

    for(i=0; i<MAX_CLIENTS; i++)
    {
        // if we haven't sent the server a message in a half second, send a keep alive.
        // otherwise the server might think we turned off.
        if(g_FrameCount - m_TickLastMessageSent[i] > 30)
        {
            m_TickLastMessageSent[i] = g_FrameCount;
            m_KeepAlive.Send(i); // send to the each individual client.
        }

        if( m_ClientAddr[i].sin_port != 0 )
        {
            // if we haven't recvd a packet from a client in 3 seconds, get rid of him.
            if(g_FrameCount - m_TickLastMessageRecvd[i] > NETWORK_TIMEOUT)
            {
                RemoveClient(i);

                int player;
                // remove any players from this client.
                for(player = 0; player < MAX_PLAYERS; player++)
                {
                    if(g_pPlayers[player] && g_pPlayers[player]->m_MachineNum == i)
                    {
                        MessagePlayerLeft* mess = MyNew(MessagePlayerLeft);
                        mess->player = player;
                        mess->Send();
                        SAFE_REDUCEREFANDDELETE(mess);
                    }
                }
            }
        }
    }

    m_Joinable = 1;
    if(m_NumClients == MAX_CLIENTS)
        m_Joinable = 0;

    static int timepassed = 0;
    timepassed++;
    if(timepassed > 60*30) // 30 seconds
    {
#if SYSTEM_NDS
        // ugly hack to update server, close, update, then recreate the socket.
        {
            close(m_ServerSock);

            int num = 0;
            for(i=0; i<MAX_CLIENTS; i++)
                if(m_ClientAddr[i].sin_port != 0)
                    num++;

            SERVERLIST_UpdateServer(m_Joinable, num);

            m_ServerSock = socket(PF_INET, SOCK_DGRAM, 0);
            bind( m_ServerSock, (sockaddr*)&m_ServerAddr, sizeof(sockaddr_in) );
            int i=1;
            ioctl(m_ServerSock, FIONBIO, &i);
        }
#endif //SYSTEM_NDS

        //SERVERLIST_UpdateServer(m_Joinable, m_NumClients);
        timepassed = 0;
    }

    return 0;
}

int NetServer::ProcessMessage(NetMessage* message, int conn)
{
    ProcessGameMessages(message, conn);
    //int type = message->type;

    //switch(type)
    //{

    //default:
    //    ProcessGameMessages(message);
    //    break;
    //}

    return 0;
}
