#ifndef __NETCODE_H__
#define __NETCODE_H__

#define MAX_CLIENTS     7

void PrintToScreen(char* text, int linein = -1, int debugonly = 1);
void PrintToScreenReset();

class NetClient
{
private:
    SOCKET              m_ServerSock;

    unsigned int        m_ServerPort;
    sockaddr_in         m_ServerAddr;
    
    PacketStack*        m_PacketStack;

    MessageKeepAlive    m_KeepAlive;
    int                 m_TickLastMessageSent;
    int                 m_TickLastMessageRecvd;

public:
    NetClient();
    ~NetClient();

    void Init(int port);
    void Connect(unsigned long ip);

    void Send(NetMessage* message, int size);

    void Close();

    int Tick();
    int ProcessMessage(NetMessage* message, int conn);
    int IsThisTheServer(sockaddr_in* sain);
};

class NetServer
{
private:
    SOCKET              m_ServerSock;
    unsigned int        m_Port;

    sockaddr_in         m_ServerAddr;
    sockaddr_in         m_ClientAddr[MAX_CLIENTS];

    PacketStack*        m_PacketStack[MAX_CLIENTS];

    int                 m_NumClients;
    int                 m_Joinable;

    MessageKeepAlive    m_KeepAlive;
    int                 m_TickLastMessageSent[MAX_CLIENTS];
    int                 m_TickLastMessageRecvd[MAX_CLIENTS];

public:
    NetServer();
    ~NetServer();

    void Init(int port);
    void CheckForIncomingConnections();
    int AddClient(sockaddr_in* sain, int CheckIfNewClient);
    int IsClientConnected(int conn);
    void RemoveClient(int conn);

    void Send(NetMessage* message, int size, int clienttoignore = -1, int clienttosendto = -1);

    void Close();

    int Tick();
    int ProcessMessage(NetMessage* message, int conn);

    void SetJoinable(int joinable) { m_Joinable = joinable; }

    int GetNumClients() { return m_NumClients; }
};

#endif //__NETCODE_H__
