#include "MySystem.h"

#include "PacketStack.h"

extern int g_FrameCount;
extern NetClient* GameClient;
extern NetServer* GameServer;

PacketStack::PacketStack()
{
    int i;

    for(i=0; i<MAX_PACKETSENT_BACKLOG; i++)
        memset( &m_MessagesAwaitingAck[i], 0, sizeof(MessageInfo) );
    m_NumMessagesAwaitingAck = 0;

    m_ConnectionNumber = -1;
    m_DestSocket = -1;

    m_NextMessageNumber = 0;

    for(i=0; i<MAX_PACKETRECEIVED_BACKLOG; i++)
    {
        m_MessagesOutOfOrder[i] = NULL;
        m_MessagesOutOfOrderID[i] = -1;
    }
    m_NumMessagesOutOfOrder = 0;
    m_NextMessageNeeded = 0;

    m_MessageAck.type = NM_Ack;
    m_MessageAckSize = sizeof(m_MessageAck);
}

PacketStack::~PacketStack()
{
    Cleanup();
}

void PacketStack::Cleanup()
{
    int i;

    m_ConnectionNumber = -1;
    m_DestSocket = -1;
    m_NextMessageNumber = 0;

    for(i=0; i<m_NumMessagesAwaitingAck; i++) //MAX_PACKETSENT_BACKLOG; i++)
    {
        //if(m_MessagesAwaitingAck[i].message)
        {
            SAFE_REDUCEREFANDDELETE(m_MessagesAwaitingAck[i].message);
        }
    }

    m_NumMessagesAwaitingAck = 0;
    m_NextMessageNumber = 0;

    for(i=0; i<m_NumMessagesOutOfOrder; i++) //MAX_PACKETRECEIVED_BACKLOG; i++)
    {
        //if(m_MessagesOutOfOrder[i])
        {
            SAFE_REDUCEREFANDDELETE(m_MessagesOutOfOrder[i]);
            m_MessagesOutOfOrderID[i] = -1;
        }
    }

    m_NumMessagesOutOfOrder = 0;
    m_NextMessageNeeded = 0;
}

void PacketStack::RemoveMessageFromStack(int id, int deletemessage)
{
    int i;
    for(i=0; i<m_NumMessagesAwaitingAck; i++)
    {
        if(m_MessagesAwaitingAck[i].id == id)
        {
            //sprintf(temptext, "Removed - %d", m_MessagesAwaitingAck[i].message->id);
            //PrintToScreen(temptext);

            SAFE_REDUCEREFANDDELETE(m_MessagesAwaitingAck[i].message);
            m_NumMessagesAwaitingAck--;
            m_MessagesAwaitingAck[i] = m_MessagesAwaitingAck[m_NumMessagesAwaitingAck];
            return;
        }
    }
}

void PacketStack::CheckForOldMessagesToResend()
{
    int i;
    for(i=0; i<m_NumMessagesAwaitingAck; i++)
    {
        if( g_FrameCount > m_MessagesAwaitingAck[i].framesent + 10 )
        {
            sockaddr_in* saddrin = (sockaddr_in*)&m_DestSain; //(sockaddr_in*)&m_MessagesAwaitingAck[i].sain;
            sprintf(temptext, "Resend - %d, %lx", m_MessagesAwaitingAck[i].id, saddrin->sin_addr.s_addr);
            //sprintf(temptext, "Resend - %d, %s", m_MessagesAwaitingAck[i].message->id, m_MessagesAwaitingAck[i].sain.sa_data);
            //sprintf(temptext, "Resend - %d", m_MessagesAwaitingAck[i].message->id);
            PrintToScreen(temptext);

            m_MessagesAwaitingAck[i].message->id = m_MessagesAwaitingAck[i].id;
            sendto( m_DestSocket, (char*)m_MessagesAwaitingAck[i].message, 
                    m_MessagesAwaitingAck[i].size, 0,
                    &m_DestSain, sizeof(sockaddr_in) );

            m_MessagesAwaitingAck[i].framesent = g_FrameCount;
        }
    }
}

int PacketStack::IsMessageInOutOfOrderQueue(int id)
{
    int i;
    for(i=0; i<m_NumMessagesOutOfOrder; i++)
    {
        if(m_MessagesOutOfOrderID[i] == id)
        {
            return 1;
        }
    }

    return 0;
}

void PacketStack::CheckForOutOfOrderMessages()
{
    int i;
    while(1)
    {
        for(i=0; i<m_NumMessagesOutOfOrder; i++)
        {
            if(m_MessagesOutOfOrderID[i] < m_NextMessageNeeded)
            {
                sprintf(temptext, "Deleting old OoO message - %d", m_MessagesOutOfOrderID[i]);
                PrintToScreen(temptext);

                SAFE_REDUCEREFANDDELETE(m_MessagesOutOfOrder[i]);
                //SAFE_DELETE(m_MessagesOutOfOrder[i]);
                //m_MessagesOutOfOrderID[i] = -1;
                m_NumMessagesOutOfOrder--;
                m_MessagesOutOfOrder[i] = m_MessagesOutOfOrder[m_NumMessagesOutOfOrder];
                m_MessagesOutOfOrderID[i] = m_MessagesOutOfOrderID[m_NumMessagesOutOfOrder];
                break;
            }

            if(m_MessagesOutOfOrderID[i] == m_NextMessageNeeded)
            {
                sprintf(temptext, "Processing OoO message - %d", m_MessagesOutOfOrderID[i]);
                PrintToScreen(temptext);

                m_NextMessageNeeded++;

                if(GameServer != NULL)
                    GameServer->ProcessMessage(m_MessagesOutOfOrder[i], m_ConnectionNumber);
                if(GameClient != NULL)
                    GameClient->ProcessMessage(m_MessagesOutOfOrder[i], m_ConnectionNumber);

                SAFE_REDUCEREFANDDELETE(m_MessagesOutOfOrder[i]);
                //SAFE_DELETE(m_MessagesOutOfOrder[i]);
                //m_MessagesOutOfOrderID[i] = -1;
                m_NumMessagesOutOfOrder--;
                m_MessagesOutOfOrder[i] = m_MessagesOutOfOrder[m_NumMessagesOutOfOrder];
                m_MessagesOutOfOrderID[i] = m_MessagesOutOfOrderID[m_NumMessagesOutOfOrder];
                break;
            }
        }

        if(i == m_NumMessagesOutOfOrder)
            return;
    }
}

int PacketStack::SendTo(int reliable, SOCKET socket, NetMessage* message, int size, int flags, sockaddr* addr, int addrsize)
{
    if(m_NumMessagesAwaitingAck < MAX_PACKETSENT_BACKLOG)
    {
        if(reliable)
        {
            message->id = m_NextMessageNumber;
            m_NextMessageNumber++;
            if(m_NextMessageNumber > 65000)
                m_NextMessageNumber = 0;

            message->refcount++;

            m_MessagesAwaitingAck[m_NumMessagesAwaitingAck].message = message;
            m_MessagesAwaitingAck[m_NumMessagesAwaitingAck].id = message->id;
            m_MessagesAwaitingAck[m_NumMessagesAwaitingAck].size = size;
            m_MessagesAwaitingAck[m_NumMessagesAwaitingAck].framesent = g_FrameCount;

            m_NumMessagesAwaitingAck++;
        }

        return sendto(socket, (char*)message, size, 0, addr, addrsize);
    }

    // TODO: return and handle too many packets in list.
    return -1;
}

void PacketStack::Tick(SOCKET socket)
{
    sprintf(temptext, "NeedAck - %d, %d", m_NumMessagesAwaitingAck, m_NextMessageNumber);
    PrintToScreen(temptext, 22);
    sprintf(temptext, "OoO - %d, %d - Tick - %d", m_NumMessagesOutOfOrder, m_NextMessageNeeded, g_FrameCount);
    PrintToScreen(temptext, 23);

    // resend old message that haven't been ack'd
    CheckForOldMessagesToResend();

    // go through out of order messages to check for new ones
    CheckForOutOfOrderMessages();
}

void PacketStack::ProcessMessage(SOCKET socket, char* message, int size, int connindex)
{
    NetMessage* nm = (NetMessage*)message;
    // this isn't allocated ram, so ignore the refcount,
    // refcount will be lost if/when we make a copy.
    //nm->refcount = 0;
    int reliable = nm->reliable;

    if(nm->type == NM_Ack)
    {
        sprintf(temptext, "Received Ack - %d %d", nm->id, nm->type);
        PrintToScreen(temptext);
        RemoveMessageFromStack(nm->id, 0);
        return;
    }
    else if(nm->type == NM_KeepAlive)
    {
    }
    else
    {
        if(!reliable)
        {
            if(GameServer != NULL)
                GameServer->ProcessMessage(nm, m_ConnectionNumber);//newmess);
            if(GameClient != NULL)
                GameClient->ProcessMessage(nm, m_ConnectionNumber);//newmess);

            if(nm->forward && GameServer)
                GameServer->Send(nm, size, connindex);
        }
        else
        {
            if(nm->id >= m_NextMessageNeeded)
            {
                sprintf(temptext, "PackRecv - %d, %d", nm->id, nm->type);
                PrintToScreen(temptext);

                int forward = 0;
                forward = nm->forward;

                NetMessage* newmess = MakeMessageCopy(nm);

                int id = newmess->id;

                if(forward && GameServer)
                {
                    // this will corrupt newmess->id... so it can't be used by the code below
                    GameServer->Send(newmess, size, connindex);
                }

                if(id == m_NextMessageNeeded)
                {
                    // process messages that are in order.
                    m_MessageAck.id = id;
                    sendto(socket, (char*)&m_MessageAck, m_MessageAckSize, 0, (sockaddr*)&m_DestSain, sizeof(sockaddr_in));
                    //sprintf(temptext, "Sent Ack - %d %d", m_MessageAck.id, m_MessageAck.type);
                    //PrintToScreen(temptext);

                    m_NextMessageNeeded++;
                    if(GameServer != NULL)
                        GameServer->ProcessMessage(newmess, m_ConnectionNumber);
                    if(GameClient != NULL)
                        GameClient->ProcessMessage(newmess, m_ConnectionNumber);
                }
                else if(id > m_NextMessageNeeded)
                {
                    // process messages that are out of order.
                    sprintf(temptext, "Recv OoO");
                    PrintToScreen(temptext);

                    if(m_NumMessagesOutOfOrder < MAX_PACKETRECEIVED_BACKLOG)
                    {
                        m_MessageAck.id = id;
                        sendto(socket, (char*)&m_MessageAck, m_MessageAckSize, 0, (sockaddr*)&m_DestSain, sizeof(sockaddr_in));
                        //sprintf(temptext, "Sent Ack - %d %d", m_MessageAck.id, m_MessageAck.type);
                        //PrintToScreen(temptext);

                        if( !IsMessageInOutOfOrderQueue(id) )
                        {
                            newmess->refcount++;
                            m_MessagesOutOfOrder[m_NumMessagesOutOfOrder] = newmess;
                            m_MessagesOutOfOrderID[m_NumMessagesOutOfOrder] = id;
                            m_NumMessagesOutOfOrder++;
                        }
                    }
                    else
                    {
                        // TODO: handle too many out of order messages, can't store more.
                    }
                }

                SAFE_REDUCEREFANDDELETE(newmess);
            }
            else
            {
                // old message we've processed is still being sent to us, Ack it to make it go away.
                m_MessageAck.id = nm->id;
                sendto(socket, (char*)&m_MessageAck, m_MessageAckSize, 0, (sockaddr*)&m_DestSain, sizeof(sockaddr_in));
                //sprintf(temptext, "Sent Ack - %d %d", m_MessageAck.id, m_MessageAck.type);
                //PrintToScreen(temptext);
            }
        }
    }
}

void PacketStack::SetConnectionInfo(int conn, SOCKET socket, sockaddr_in* addr)
{
    m_ConnectionNumber = conn;
    m_DestSocket = socket;
    m_DestSain = *((sockaddr*)addr);
}
