#include "MySystem.h"
#include "TileInfo.h"
#include "MyDMA.h"

#include "SpriteRamHandler.h"

SpriteManager g_MainSprites;
SpriteManager g_SubSprites;

int NumSpritesUsed = 0;

int SpritesAdded = 0;

void LoadPalette(u16* PaletteMem, int offset, u16* palette, int numcolors)
{
    DMA_32bitCopy(3, (uint32*)palette, (uint32*)PaletteMem+offset/2, numcolors/2);
}

SpriteManager::SpriteManager()
{
    int i;

    for(i=0; i<128; i++)
    {
        m_SpritesOAM[i].m_Sprite.attribute[0] = ATTR0_DISABLED;
        m_SpritesOAM[i].m_Sprite.attribute[1] = 0;
        m_SpritesOAM[i].m_Sprite.attribute[2] = 0;
        //m_SortOrder[i] = 0;
    }

    m_numSprites = 0;

    m_MemOffset = 0;
    m_TileOffset = 0;

    m_TileStep = 32;
}

SpriteManager::~SpriteManager()
{
}

void SpriteManager::ResetLoadedTiles(void)
{
    m_MemOffset = 0;
    m_TileOffset = 0;
}

void SpriteManager::SetTileStepSize(int step)
{
    m_TileStep = step;
}

void SpriteManager::ManualTileOffset(int mem)
{
    m_MemOffset += mem;
    m_TileOffset = m_MemOffset/m_TileStep; //256;
}

void SpriteManager::LoadTiles(u16* TileMem, TilesInfo* TI, bool OffsetTiles)
{
    int i;

    u16 memsize = TI->MemOffset[TI->numFrames];

    memcpy((uint32*)(TileMem+m_MemOffset/2), (uint32*)TI->Tiles, memsize);
    //DMA_32bitCopy(3, (uint32*)TI->Tiles, (uint32*)(TileMem+m_MemOffset/2), memsize);

    if(OffsetTiles)
    {
        for(i=0; i<=TI->numFrames; i++)
        {
            if(TI->numColors == 16)
                TI->Offset[i] /= 2;

            // adjust for m_TileStep // instead of 64. // exporter sets offset to 1 byte per pixel.
            float divisor = m_TileStep/64.0f;
            TI->Offset[i] = (int)(TI->Offset[i] / divisor);

            TI->Offset[i] += m_TileOffset;
        }
    }

    m_MemOffset += memsize;
    m_TileOffset = m_MemOffset/m_TileStep; //256; //128;
    //m_TileOffset = m_MemOffset/128;

    static int totalram = 0;
    totalram += memsize;

    //sprintf(DEBUG_String, "memo:%d tileo:%d", m_MemOffset, m_TileOffset);
    //DEBUG_Print();
    //sprintf(DEBUG_String, "memoffset32:%x", (uint32*)(TileMem+m_MemOffset/2));
    //DEBUG_Print();
    ////sprintf(DEBUG_String, "TIMemOffset:%d", TI->Offset[TI->numFrames]);
    ////DEBUG_Print();
    ////sprintf(DEBUG_String, "TIOffset:%d", TI->Offset[0]);
    ////DEBUG_Print();
    //sprintf(DEBUG_String, "totalram:%d  new:%d", totalram, memsize);
    //DEBUG_Print();
}

void SpriteManager::ClearSprites(void)
{
    DisableAll();
    m_numSprites = 0;
    //int i;
    //for(i=0; i<256+64; i++)
    //    m_SpriteBuckets[i].NewList();
    //for(i=0; i<MAX_SPRITES; i++)
    //    m_SpritesOAM->Remove();
    SpritesAdded = 0;
}

void SpriteManager::DisableAll(void)
{
    int i;

    for(i=0; i<128; i++)
    {
        if(m_SpritesOAM[i].Next && m_SpritesOAM[i].Prev)
            m_SpritesOAM[i].Remove();
        m_SpritesOAM[i].m_Sprite.attribute[0] = ATTR0_DISABLED;
        MoveSprite(i, -64, -64);
    }
}

void SpriteManager::UpdateAll(u16* oam)
{
    int i, j;
    int x;

    int numset = 0;

    // copy our oam to the actual oam.
    if(1) // sort them from the sprite buckets.
    {
        for(x = 255; x >= -64; x--)
        {
            CPPListNode* node;
            for(node = m_SpriteBuckets[x+64].HeadNode.Next; node->Next; node = node->Next)
            {
                OAMEntry* oamentry = (OAMEntry*)node;
                for(j=0; j<2; j++)
                {
                    ((uint32*)oam)[numset] = ((uint32*)&oamentry->m_Sprite)[j];
                    numset++;
                }
            }
        }
    }
    else
    {
        for(i=0; i<m_numSprites; i++)
        {
            for(j=0; j<2; j++)
            {
                ((uint32*)oam)[numset] = ((uint32*)&m_SpritesOAM[i].m_Sprite)[j];
                numset++;
            }
        }
    }

    NumSpritesUsed = numset/2;
    //sprintf(DEBUG_String, "numsprites:%d, %d", numset/2, SpritesAdded);
    //DEBUG_Print();

    for(i=m_numSprites; i<MAX_SPRITES; i++)
    {
        ((uint32*)oam)[numset] = ATTR0_DISABLED;
        numset+=2;
    }
}

void SpriteManager::DisableSprite(int spritenum)
{
    m_SpritesOAM[spritenum].m_Sprite.attribute[0] = ATTR0_DISABLED;
    MoveSprite(spritenum, -64, -64);
}

void SpriteManager::AddSprite(int sortorder, u16 attr0, u16 attr1, u16 attr2, int x, int y)
{
    SpritesAdded++;

    if(m_numSprites == MAX_SPRITES)
    {
        DEBUG_Print("Hit Sprite Limit");
        return;
    }

    //m_SortOrder[m_numSprites] = sortorder;
    m_SpriteBuckets[sortorder + 64].AddHead(&m_SpritesOAM[m_numSprites]);
    m_SpritesOAM[m_numSprites].m_Sprite.attribute[0] = attr0;  
    m_SpritesOAM[m_numSprites].m_Sprite.attribute[1] = attr1;
    m_SpritesOAM[m_numSprites].m_Sprite.attribute[2] = attr2;

    MoveSprite(m_numSprites, x, y);
    m_numSprites++;
}

void SpriteManager::AddSprite(int sortorder, TilesInfo* TI, int frame, int posx, int posy, int transparent, int palettenum, int priority, int fliphor, int flipvert)
{
    SpritesAdded++;

    if(m_numSprites == MAX_SPRITES)
    {
        DEBUG_Print("Hit Sprite Limit");
        return;
    }

    //m_SortOrder[m_numSprites] = sortorder;
    m_SpriteBuckets[sortorder + 64].AddHead(&m_SpritesOAM[m_numSprites]);

    if(TI->numColors == 16)
        m_SpritesOAM[m_numSprites].m_Sprite.attribute[0] = ATTR0_COLOR_16 | TI->GBAAttr0[frame]<<14;  
    else
        m_SpritesOAM[m_numSprites].m_Sprite.attribute[0] = ATTR0_COLOR_256 | TI->GBAAttr0[frame]<<14;  

    if(transparent)
        m_SpritesOAM[m_numSprites].m_Sprite.attribute[0] |= ATTR0_TYPE_BLENDED;

    m_SpritesOAM[m_numSprites].m_Sprite.attribute[1] = TI->GBAAttr1[frame]<<14 | fliphor << 12 | flipvert << 13;

    m_SpritesOAM[m_numSprites].m_Sprite.attribute[2] = TI->Offset[frame] | ATTR2_PRIORITY(priority);

    int x,y;
    if(fliphor)
        x = posx - TI->OffsetX[frame] + TI->OrigWidth[frame] - TI->GBAWidth[frame];
    else
        x = posx + TI->OffsetX[frame];

    if(flipvert)
        y = posy - TI->OffsetY[frame] + TI->OrigHeight[frame] - TI->GBAHeight[frame];
    else
        y = posy + TI->OffsetY[frame];

    if(x < -64) x = -64;
    if(x > 320) x = 320;
    if(y < -64) y = -64;
    if(y > 192) y = 192;

    if(TI->numColors == 16)
    {
        //if(palettenum == -1)
        //    SetPalette(m_numSprites, palettenum); //TI->PaletteNumber);
        //else
        SetPalette(m_numSprites, TI->PaletteNumber);
    }
    else
    {
        // TODO: fix this for team nature... or give team nature a more normal number
        if(palettenum > 16)
            palettenum = 2;
        SetPalette(m_numSprites, palettenum);
    }

    MoveSprite(m_numSprites, x, y);
    m_numSprites++;
}

void SpriteManager::MoveSprite(int spritenum, int x, int y)
{
    m_SpritesOAM[spritenum].m_Sprite.attribute[1] &= 0xFE00;
    m_SpritesOAM[spritenum].m_Sprite.attribute[1] |= (x & 0x01FF);
 
    m_SpritesOAM[spritenum].m_Sprite.attribute[0] &= 0xFF00;
    m_SpritesOAM[spritenum].m_Sprite.attribute[0] |= (y & 0x00FF);
} 

void SpriteManager::SetPalette(int spritenum, int palettenum)
{
    m_SpritesOAM[spritenum].m_Sprite.attribute[2] |= ATTR2_PALETTE(palettenum);
}
