#ifndef __SPRITERAMHANDLER_H__
#define __SPRITERAMHANDLER_H__

#include "TileInfo.h"

void LoadPalette(u16* PaletteMem, int offset, u16* palette, int numcolors);
void LoadTiles(u16* TileMem, TilesInfo* TI, bool OffsetTiles);
void ResetLoadedTiles(void);

#define MAX_SPRITES     128

extern int NumSpritesUsed;

class OAMEntry : public CPPListNode
{
public:
    sSpriteEntry m_Sprite;
};

class SpriteManager
{
private:
    OAMEntry m_SpritesOAM[MAX_SPRITES];
    CPPListHead m_SpriteBuckets[256+64];

    u8 m_numSprites;

    int m_MemOffset;
    int m_TileOffset;
    int m_TileStep;

public:
    SpriteManager();
    ~SpriteManager();

    void SetTileStepSize(int step);
    void ResetLoadedTiles(void);
    void ManualTileOffset(int mem);
    void LoadTiles(u16* TileMem, TilesInfo* TI, bool OffsetTiles);

    void ClearSprites(void);

    void DisableAll(void);
    void UpdateAll(u16* oam);

    void DisableSprite(int spritenum);

    void AddSprite(int sortorder, u16 attr0, u16 attr1, u16 attr2, int x, int y);
    void AddSprite(int sortorder, TilesInfo* TI, int frame, int posx, int posy, int transparent, int palettenum, int priority, int fliphor, int flipvert);

    void MoveSprite(int spritenum, int x, int y);

    void SpriteManager::SetPalette(int spritenum, int palettenum);
};

extern SpriteManager g_MainSprites;
extern SpriteManager g_SubSprites;

#endif //__SPRITERAMHANDLER_H__
